# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'preferencesDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogPreferences(object):
    def setupUi(self, DialogPreferences):
        DialogPreferences.setObjectName("DialogPreferences")
        DialogPreferences.resize(507, 482)
        self.gridLayout_3 = QtWidgets.QGridLayout(DialogPreferences)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButtonCancel = QtWidgets.QPushButton(DialogPreferences)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout_2.addWidget(self.pushButtonCancel)
        self.pushButtonAccept = QtWidgets.QPushButton(DialogPreferences)
        self.pushButtonAccept.setObjectName("pushButtonAccept")
        self.horizontalLayout_2.addWidget(self.pushButtonAccept)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 5, 0, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(DialogPreferences)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 487, 431))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.groupBoxMisc = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxMisc.setObjectName("groupBoxMisc")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBoxMisc)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.checkBoxAutomaticOpenPreviousExperiment = QtWidgets.QCheckBox(self.groupBoxMisc)
        self.checkBoxAutomaticOpenPreviousExperiment.setObjectName("checkBoxAutomaticOpenPreviousExperiment")
        self.gridLayout_4.addWidget(self.checkBoxAutomaticOpenPreviousExperiment, 0, 0, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxMisc, 2, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem1, 3, 0, 1, 1)
        self.groupBoxWorkspace = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxWorkspace.setObjectName("groupBoxWorkspace")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBoxWorkspace)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.LineEditFilePath = QtWidgets.QLineEdit(self.groupBoxWorkspace)
        self.LineEditFilePath.setObjectName("LineEditFilePath")
        self.gridLayout_2.addWidget(self.LineEditFilePath, 0, 0, 1, 1)
        self.ButtonBrowseWorkingDir = QtWidgets.QPushButton(self.groupBoxWorkspace)
        self.ButtonBrowseWorkingDir.setObjectName("ButtonBrowseWorkingDir")
        self.gridLayout_2.addWidget(self.ButtonBrowseWorkingDir, 0, 1, 1, 1)
        self.gridLayout_5.addWidget(self.groupBoxWorkspace, 0, 0, 1, 1)
        self.groupBoxPlugins = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxPlugins.setObjectName("groupBoxPlugins")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxPlugins)
        self.formLayout.setObjectName("formLayout")
        self.pushButtonPlugins = QtWidgets.QPushButton(self.groupBoxPlugins)
        self.pushButtonPlugins.setObjectName("pushButtonPlugins")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.pushButtonPlugins)
        self.gridLayout_5.addWidget(self.groupBoxPlugins, 1, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_3.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(DialogPreferences)
        self.pushButtonCancel.clicked.connect(DialogPreferences.reject)
        self.pushButtonAccept.clicked.connect(DialogPreferences.accept)
        QtCore.QMetaObject.connectSlotsByName(DialogPreferences)

    def retranslateUi(self, DialogPreferences):
        _translate = QtCore.QCoreApplication.translate
        DialogPreferences.setWindowTitle(_translate("DialogPreferences", "Meggie - Preferences"))
        self.pushButtonCancel.setText(_translate("DialogPreferences", "Cancel"))
        self.pushButtonAccept.setText(_translate("DialogPreferences", "Ok"))
        self.groupBoxMisc.setTitle(_translate("DialogPreferences", "Miscellaneous:"))
        self.checkBoxAutomaticOpenPreviousExperiment.setText(_translate("DialogPreferences", "Automatically open previous experiment upon application startup"))
        self.groupBoxWorkspace.setTitle(_translate("DialogPreferences", "Workspace:"))
        self.ButtonBrowseWorkingDir.setText(_translate("DialogPreferences", "Browse..."))
        self.groupBoxPlugins.setTitle(_translate("DialogPreferences", "Plugins:"))
        self.pushButtonPlugins.setText(_translate("DialogPreferences", "Select active plugins..."))



""" Contains logic for the epoch creation dialog.
"""

import logging

from copy import deepcopy

from PyQt5 import QtWidgets

import numpy as np

from meggie.actions.epochs_create.dialogs.createEpochsFromEventsDialogUi import Ui_CreateEpochsFromEventsDialog
from meggie.actions.epochs_create.controller.epoching import create_epochs_from_events

from meggie.utilities.widgets.batchingWidgetMain import BatchingWidget
from meggie.utilities.dialogs.bitSelectionDialogMain import BitSelectionDialog

from meggie.utilities.validators import validate_name
from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.messaging import messagebox


class CreateEpochsFromEventsDialog(QtWidgets.QDialog):
    """Dialog responsible for collecting parameters for epoch creation.
    """

    def __init__(self, experiment, parent, default_name, handler):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_CreateEpochsFromEventsDialog()
        self.ui.setupUi(self)

        self.parent = parent
        self.experiment = experiment
        self.handler = handler

        self.events = []

        self.batching_widget = BatchingWidget(
            experiment_getter=self._experiment_getter,
            parent=self,
            container=self.ui.groupBoxBatching,
            geometry=self.ui.batchingWidgetPlaceholder.geometry())
        self.ui.gridLayoutBatching.addWidget(self.batching_widget, 0, 0, 1, 1)

        self.ui.lineEditCollectionName.setText(default_name)

    def _experiment_getter(self):
        return self.experiment

    def _update_events(self):
        self.ui.listWidgetEvents.clear()

        events = self.events

        for event in events:
            item = QtWidgets.QListWidgetItem(
                '%s, %s' % (
                    'ID ' + str(event['event_id']),
                    'mask=' + str(event['mask'])
                ))
            self.ui.listWidgetEvents.addItem(item)

    def _collect_parameter_values(self):
        tmin = float(self.ui.doubleSpinBoxTmin.value())
        tmax = float(self.ui.doubleSpinBoxTmax.value())
        bstart = float(self.ui.doubleSpinBoxBaselineStart.value())
        bend = float(self.ui.doubleSpinBoxBaselineEnd.value())
        delay = float(self.ui.doubleSpinBoxDelay.value())

        mag = self.ui.checkBoxMag.isChecked()
        grad = self.ui.checkBoxGrad.isChecked()
        eeg = self.ui.checkBoxEeg.isChecked()

        collection_name = validate_name(
            str(self.ui.lineEditCollectionName.text()))

        reject = dict()
        if mag:
            reject['mag'] = self.ui.doubleSpinBoxMagReject.value()
        if grad:
            reject['grad'] = self.ui.doubleSpinBoxGradReject.value()
        if eeg:
            reject['eeg'] = self.ui.doubleSpinBoxEEGReject.value()

        params = {'mag': mag, 'grad': grad, 'eeg': eeg,
                  'reject': reject,
                  'tmin': tmin, 'tmax': tmax,
                  'bstart': bstart, 'bend': bend,
                  'delay': delay,
                  'collection_name': collection_name,
                  'events': self.events}
        return params

    def accept(self):
        if self.ui.listWidgetEvents.count() == 0:
            message = 'Cannot create epochs from empty list.'
            messagebox(self.parent, message)
            return

        try:
            params = self._collect_parameter_values()
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        subject = self.experiment.active_subject

        try:
            self.handler(subject, params)
            self.experiment.save_experiment_settings()
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        self.parent.initialize_ui()

        self.close()

    def acceptBatch(self):
        experiment = self.experiment

        if self.ui.listWidgetEvents.count() == 0:
            message = 'Cannot create epochs from empty list.'
            messagebox(self.parent, message)
            return

        try:
            params = self._collect_parameter_values()
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        selected_subject_names = self.batching_widget.selected_subjects
        for name, subject in self.experiment.subjects.items():
            if name in selected_subject_names:
                try:
                    self.handler(subject, params)
                    subject.release_memory()
                except Exception as exc:
                    self.batching_widget.failed_subjects.append((subject,
                                                                 str(exc)))
                    logging.getLogger('ui_logger').exception('')

        self.batching_widget.cleanup()

        try:
            self.experiment.save_experiment_settings()
        except Exception as exc:
            exc_messagebox(self.parent, exc)

        self.parent.initialize_ui()
        self.close()

    def on_pushButtonEdit_clicked(self, checked=None):
        if checked is None:
            return

        self.bitDialog = BitSelectionDialog(self,
                                            self.ui.spinBoxMask, self.ui.spinBoxEventID)

        self.bitDialog.show()

    def on_pushButtonHelp_clicked(self, checked=None):
        if checked is None:
            return

        help_message = (
            "Events are found in a following way. If only event "
            "id is set, events with exactly the same binary representation as "
            "event id are included in the final event list. If also mask is "
            "set, event list will also include events where binary digits in "
            "the places specified by the mask are not the same as in the "
            "event id, or in other words, only events where the digits we "
            "are interested in are the same as in the list of all events, "
            "are included. Binary representations are assumed to be 16 digits "
            "long. \n\nFor example event id of 0000010000010000 = 1040 and "
            "mask of 0000000000000011 = 3 would mean that first (rightmost) "
            "two digits can be 1 or 0, but anything else must be exactly as "
            "in the event id. Thus events with following id's would be allowed:"
            "\n\n0000010000010000 = 1040\n0000010000010001 = 1041\n"
            "0000010000010010 = 1042\n0000010000010011 = 1043")

        messagebox(self.parent, help_message, 'Mask help')

    def on_pushButtonAdd_clicked(self, checked=None):
        if checked is None:
            return

        event_params = {
            'mask': self.ui.spinBoxMask.value(),
            'event_id': self.ui.spinBoxEventID.value(),
        }

        if event_params not in self.events:
            self.events.append(event_params)
            self._update_events()

    def on_pushButtonClear_clicked(self, checked=None):
        if checked is None:
            return

        self.events = []
        self._update_events()




from meggie.utilities.plotting import color_cycle

def test_color_cycle():
    colors = color_cycle(30)
    assert(type(colors) == list)
    assert(len(colors) == 30)
    assert(len(set(colors)) == 8)




""" Contains a class for logic of channel groups dialog.
"""

import os
import logging

from copy import deepcopy

from PyQt5 import QtCore
from PyQt5 import QtWidgets

import mne
import matplotlib as mpl
import matplotlib.pyplot as plt

import meggie.utilities.filemanager as filemanager

from meggie.utilities.messaging import messagebox
from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.channels import get_default_channel_groups
from meggie.utilities.channels import get_triplet_from_mag

from meggie.mainwindow.dialogs.channelGroupsDialogUi import Ui_channelGroupsDialog


class ChannelGroupsDialog(QtWidgets.QDialog):
    """ Contains the the logic for channel groups dialog.
    """

    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_channelGroupsDialog()
        self.ui.setupUi(self)

        self.parent = parent

        channel_groups = self.parent.experiment.channel_groups

        self.eeg_channel_groups = deepcopy(channel_groups['eeg'])
        self.meg_channel_groups = deepcopy(channel_groups['meg'])

        # default channel type is MEG
        for ch_name in sorted(self.meg_channel_groups.keys()):
            self.ui.listWidgetChannelGroups.addItem(ch_name)

    def on_pushButtonAdd_clicked(self, checked=None):
        if checked is None:
            return

        name = self.ui.lineEditAdd.text()

        if self.ui.radioButtonEEG.isChecked():
            if name in self.eeg_channel_groups.keys():
                return
            self.eeg_channel_groups[name] = []

            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.eeg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)
        else:
            if name in self.meg_channel_groups.keys():
                return
            self.meg_channel_groups[name] = []

            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.meg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)

    def on_pushButtonRemove_clicked(self, checked=None):
        if checked is None:
            return

        if not self.ui.listWidgetChannelGroups.currentItem():
            return

        current_item = str(self.ui.listWidgetChannelGroups.currentItem().text())

        if self.ui.radioButtonEEG.isChecked():
            self.eeg_channel_groups.pop(current_item)
            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.eeg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)
        else:
            self.meg_channel_groups.pop(current_item)
            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.meg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)


    def on_pushButtonReset_clicked(self, checked=None):
        if checked is None:
            return

        subject = self.parent.experiment.active_subject
        if not subject:
            messagebox(self.parent, 'To reset, active subject is needed')
            return

        info = subject.get_raw().info

        if self.ui.radioButtonEEG.isChecked():
            self.eeg_channel_groups = get_default_channel_groups(info, 'eeg')
            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.eeg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)

        else:
            meg_defaults = get_default_channel_groups(info, 'meg')
            self.meg_channel_groups = meg_defaults
            self.ui.listWidgetChannelGroups.clear()
            for ch_name in sorted(self.meg_channel_groups.keys()):
                self.ui.listWidgetChannelGroups.addItem(ch_name)

    def on_pushButtonSetChannels_clicked(self, checked=None):
        if checked is None:
            return

        subject = self.parent.experiment.active_subject
        if not subject:
            messagebox(self.parent, 'To set channels, active subject is needed')
            return

        try:
            selected_item = self.ui.listWidgetChannelGroups.selectedItems()[0]
        except IndexError:
            messagebox(self.parent, 'Select a channel group first')
            return

        info = subject.get_raw().info
        
        if self.ui.radioButtonEEG.isChecked():
            if mne.pick_types(info, meg=False, eeg=True).size == 0:
                messagebox(self.parent, 'No EEG channels found')
                return

            ch_names = self.eeg_channel_groups[selected_item.text()]
            try:
                ch_idxs = [info['ch_names'].index(ch_name) for ch_name in ch_names]
            except ValueError:
                messagebox(self.parent, "Channel names in the group "
                                        "and the current subject don't match")
                return

            ch_groups = [[], ch_idxs]

            try:
                fig, selection = mne.viz.plot_sensors(info, kind='select', ch_type='eeg',
                                                      ch_groups=ch_groups, show=False,
                                                      title='Group channels')
            except RuntimeError:
                messagebox(self.parent, 'Could not plot sensors. Is the montage set?')
                return

        else:
            if mne.pick_types(info, meg=True, eeg=False).size == 0:
                messagebox(self.parent, 'No MEG channels found')
                return

            ch_names = self.meg_channel_groups[selected_item.text()]
            try:
                ch_idxs = [info['ch_names'].index(ch_name) for ch_name in ch_names]
            except ValueError:
                messagebox(self.parent, "Channel names in the group "
                                        "and the current subject don't match")
                return

            ch_groups = [[], ch_idxs]

            fig, selection = mne.viz.plot_sensors(info, kind='select', ch_type='mag',
                                                  ch_groups=ch_groups, show=False,
                                                  title='Group channels')

        # make markers bigger
        for child in fig.axes[0].get_children():
            if isinstance(child, mpl.collections.PathCollection):
                child.set_sizes([100])
                child.set_lw([1])
                break

        plt.show()

        def on_close(event):
            if self.ui.radioButtonMEG.isChecked():
                if selection:
                    all_megs = []
                    for ch_name in selection:
                        all_megs.extend(get_triplet_from_mag(ch_name))
                    self.meg_channel_groups[selected_item.text()] = all_megs
            else:
                if selection:
                    self.eeg_channel_groups[selected_item.text()] = selection


        fig.canvas.mpl_connect('close_event', on_close)

    def on_radioButtonEEG_toggled(self):
        self.ui.listWidgetChannelGroups.clear()
        for ch_name in sorted(self.eeg_channel_groups.keys()):
            self.ui.listWidgetChannelGroups.addItem(ch_name)

    def on_radioButtonMEG_toggled(self):
        self.ui.listWidgetChannelGroups.clear()
        for ch_name in sorted(self.meg_channel_groups.keys()):
            self.ui.listWidgetChannelGroups.addItem(ch_name)

    def accept(self):
        channel_groups = {
            'eeg': self.eeg_channel_groups,
            'meg': self.meg_channel_groups
        }
        try:
            self.parent.experiment.channel_groups = channel_groups
            self.parent.experiment.save_experiment_settings()
        except Exception as exc:
            exc_messagebox(self, exc)
            return
        
        logging.getLogger('ui_logger').info('Channel groups saved.')
        self.close()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'createExperimentDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_CreateExperimentDialog(object):
    def setupUi(self, CreateExperimentDialog):
        CreateExperimentDialog.setObjectName("CreateExperimentDialog")
        CreateExperimentDialog.setWindowModality(QtCore.Qt.WindowModal)
        CreateExperimentDialog.resize(506, 373)
        self.gridLayout_3 = QtWidgets.QGridLayout(CreateExperimentDialog)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.scrollArea = QtWidgets.QScrollArea(CreateExperimentDialog)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 488, 324))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)
        self.groupBoxInfo = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxInfo.setObjectName("groupBoxInfo")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBoxInfo)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelExperimentName = QtWidgets.QLabel(self.groupBoxInfo)
        self.labelExperimentName.setObjectName("labelExperimentName")
        self.gridLayout_2.addWidget(self.labelExperimentName, 0, 0, 1, 1)
        self.lineEditExperimentName = QtWidgets.QLineEdit(self.groupBoxInfo)
        self.lineEditExperimentName.setObjectName("lineEditExperimentName")
        self.gridLayout_2.addWidget(self.lineEditExperimentName, 0, 1, 1, 1)
        self.lineEditAuthor = QtWidgets.QLineEdit(self.groupBoxInfo)
        self.lineEditAuthor.setObjectName("lineEditAuthor")
        self.gridLayout_2.addWidget(self.lineEditAuthor, 1, 1, 1, 1)
        self.labelAuthor = QtWidgets.QLabel(self.groupBoxInfo)
        self.labelAuthor.setObjectName("labelAuthor")
        self.gridLayout_2.addWidget(self.labelAuthor, 1, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBoxInfo, 0, 1, 1, 1)
        self.groupBoxPipeline = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxPipeline.setObjectName("groupBoxPipeline")
        self.gridLayoutPipeline = QtWidgets.QGridLayout(self.groupBoxPipeline)
        self.gridLayoutPipeline.setObjectName("gridLayoutPipeline")
        self.gridLayout.addWidget(self.groupBoxPipeline, 1, 1, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_3.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayoutButtons = QtWidgets.QHBoxLayout()
        self.horizontalLayoutButtons.setObjectName("horizontalLayoutButtons")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayoutButtons.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(CreateExperimentDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayoutButtons.addWidget(self.pushButtonCancel)
        self.pushButtonAccept = QtWidgets.QPushButton(CreateExperimentDialog)
        self.pushButtonAccept.setObjectName("pushButtonAccept")
        self.horizontalLayoutButtons.addWidget(self.pushButtonAccept)
        self.gridLayout_3.addLayout(self.horizontalLayoutButtons, 2, 0, 1, 1)

        self.retranslateUi(CreateExperimentDialog)
        self.pushButtonAccept.clicked.connect(CreateExperimentDialog.accept)
        self.pushButtonCancel.clicked.connect(CreateExperimentDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(CreateExperimentDialog)

    def retranslateUi(self, CreateExperimentDialog):
        _translate = QtCore.QCoreApplication.translate
        CreateExperimentDialog.setWindowTitle(_translate("CreateExperimentDialog", "Meggie - Create new experiment"))
        self.groupBoxInfo.setTitle(_translate("CreateExperimentDialog", "Experiment information"))
        self.labelExperimentName.setText(_translate("CreateExperimentDialog", "Experiment name:"))
        self.labelAuthor.setText(_translate("CreateExperimentDialog", "Experiment author:"))
        self.groupBoxPipeline.setTitle(_translate("CreateExperimentDialog", "Select a pipeline for the analysis:"))
        self.pushButtonCancel.setText(_translate("CreateExperimentDialog", "Cancel"))
        self.pushButtonAccept.setText(_translate("CreateExperimentDialog", "Ok"))



import os 

import mne
import numpy as np

from meggie.utilities.events import find_events
from meggie.utilities.events import find_stim_channel
from meggie.utilities.events import update_stim_channel


def test_find_events():
    sample_folder = mne.datasets.sample.data_path()
    sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')
    raw = mne.io.read_raw_fif(sample_fname, preload=True)

    id_ = 1
    mask = 32 + 16 + 8 + 4 + 2 + 1
    events_all_with_mask = find_events(raw, mask=mask, id_=1)

    events_all = find_events(raw)

    events_1 = find_events(raw, id_=1)

    assert(len(events_all_with_mask) == len(events_all) == 320)
    assert(len(events_1) == 72)
    assert(np.array_equal(events_1[:, 2], [1]*len(events_1)))


def test_find_stim_channel():
    sample_folder = mne.datasets.sample.data_path()
    sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')
    raw = mne.io.read_raw_fif(sample_fname, preload=True)

    stim_channel = find_stim_channel(raw)
    assert(stim_channel == 'STI 014')


def test_update_stim_channel():
    sample_folder = mne.datasets.sample.data_path()
    sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')
    raw = mne.io.read_raw_fif(sample_fname, preload=True)

    events_before = find_events(raw)

    update_stim_channel(raw, [[100, 0, 1]])

    events_after = find_events(raw)

    assert(len(events_after) == len(events_before) + 1)




""" Contains controlling logic for the tfr group average
"""

import logging

import mne

from meggie.utilities.channels import clean_names
from meggie.utilities.threading import threaded
from meggie.utilities.validators import assert_arrays_same

from meggie.datatypes.tfr.tfr import TFR


@threaded
def group_average_tfr(experiment, tfr_name, groups, new_name):
    """ Computes a group average item."""

    # check data cohesion
    keys = []
    freq_arrays = []
    time_arrays = []
    for group_key, group_subjects in groups.items():
        for subject_name in group_subjects:
            try:
                subject = experiment.subjects.get(subject_name)
                tfr = subject.tfr.get(tfr_name)
                keys.append(tuple(sorted(tfr.content.keys())))
                freq_arrays.append(tuple(tfr.freqs))
                time_arrays.append(tuple(tfr.times))
            except Exception as exc:
                continue

    assert_arrays_same(keys, 'Conditions do no match')
    assert_arrays_same(freq_arrays, 'Freqs do not match')
    assert_arrays_same(time_arrays)

    # handle channel differences
    ch_names = []
    for group_key, group_subjects in groups.items():
        for subject_name in group_subjects:
            try:
                subject = experiment.subjects.get(subject_name)
                tfr = subject.tfr.get(tfr_name)
                ch_names.append(tuple(clean_names(tfr.ch_names)))
            except Exception as exc:
                continue

    if len(set(ch_names)) != 1:
        logging.getLogger('ui_logger').info(
            "TFR's contain different sets of channels. Identifying common ones..")

        common_ch_names = list(set.intersection(*map(set, ch_names)))

        logging.getLogger('ui_logger').info(str(len(common_ch_names)) +
                                            ' common channels found.')
    else:
        common_ch_names = ch_names[0]

    grand_tfrs = {}
    for group_key, group_subjects in groups.items():
        for subject in experiment.subjects.values():
            if subject.name not in group_subjects:
                continue

            meggie_tfr = subject.tfr.get(tfr_name)
            if not meggie_tfr:
                continue

            for tfr_item_key, tfr_item in meggie_tfr.content.items():
                grand_key = (group_key, tfr_item_key)

                # get common channels in "subject specific space"
                subject_ch_names = tfr_item.info['ch_names']
                for ch_idx, ch_name in enumerate(clean_names(subject_ch_names)):
                    drop_names = []
                    if ch_name not in common_ch_names:
                        drop_names.append(subject_ch_names[ch_idx])
                tfr_item = tfr_item.copy().drop_channels(drop_names)

                # sanity check
                if len(tfr_item.info['ch_names']) != len(common_ch_names):
                    raise Exception('Something wrong with the channels')

                if grand_key in grand_tfrs:
                    grand_tfrs[grand_key].append(tfr_item)
                else:
                    grand_tfrs[grand_key] = [tfr_item]

    grand_averages = {}
    for key, grand_tfr in grand_tfrs.items():
        new_key = str(key[1]) + '_group_' + str(key[0])
        if len(grand_tfr) == 1:
            grand_averages[new_key] = grand_tfr[0].copy()
        else:
            grand_averages[new_key] = mne.grand_average(
                grand_tfr)

    active_subject = experiment.active_subject
    meggie_tfr = active_subject.tfr.get(tfr_name)

    params = {
        'decim': meggie_tfr.decim,
        'n_cycles': meggie_tfr.n_cycles,
        'evoked_subtracted': meggie_tfr.evoked_subtracted,
        'conditions': list(grand_averages.keys()),
        'groups': groups
    }

    meggie_tfr = TFR(new_name, active_subject.tfr_directory, params, grand_averages)

    meggie_tfr.save_content()
    active_subject.add(meggie_tfr, "tfr")




"""Defines an abstract class to derive from when creating datatypes."""


class Datatype:
    """Abstract class that shows the methods that can be expected to 
    be present in the actual datatypes.

    Parameters
    ----------
    name : str
        Name of the data object, used in the UI lists and in the .exp file.
    directory : str
        Absolute path to the folder where the data is expected to be stored.
    params : dict
        Contains additional information about the data object.
    content : dict or instance, optional
        Stores the actual numerical data.
        If not provided, it is assumed that the content is already stored 
        in the file system.
    """
    def __init__(self, name, directory, params, content=None):
        pass
 
    def save_content(self):
        """Saves the data object contents to the file system."""
        raise NotImplementedError('')

    def delete_content(self):
        """Deletes the data object contents from the file system."""
        raise NotImplementedError('')

    @property
    def content(self):
        """Returns the actual stored contents. Is expected to load
        it from the file system if not loaded already."""
        raise NotImplementedError('')

    @property
    def data(self):
        """In contrast to returning contents, that can be for example mne.Epochs,
        return actual numpy arrays """
        raise NotImplementedError('')

    @property
    def info(self):
        """Returns info, usually needed for sensor locations."""
        raise NotImplementedError('')

    @property
    def params(self):
        """Returns related information about the data object,
        stored in the experiment file."""
        raise NotImplementedError('')

    @property
    def times(self):
        """If time-based object, should return the times."""
        raise NotImplementedError('')

    @property
    def freqs(self):
        """If freqs-based object, should return the freqs."""
        raise NotImplementedError('')

    @property
    def ch_names(self):
        """If contains channels, should return the ch_names."""
        raise NotImplementedError('')



""" Contains a class for logic of the add advanced dialog.
"""
import logging

from PyQt5 import QtWidgets

from meggie.utilities.dialogs.powerSpectrumAddAdvancedDialogUi import Ui_PowerSpectrumAddAdvancedDialog
from meggie.utilities.dialogs.bitSelectionDialogMain import BitSelectionDialog

from meggie.utilities.messaging import messagebox


class PowerSpectrumAddAdvancedDialog(QtWidgets.QDialog):
    """ Contains logic of the add advanced dialog.
    """

    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.intervals = []
        self.ui = Ui_PowerSpectrumAddAdvancedDialog()
        self.ui.setupUi(self)
        self.parent = parent

    def on_pushButtonStartEdit_clicked(self, checked=None):
        if checked is None:
            return
        self.bitDialog = BitSelectionDialog(self, self.ui.spinBoxStartMask,
                                            self.ui.spinBoxStartId)
        self.bitDialog.show()

    def on_pushButtonEndEdit_clicked(self, checked=None):
        if checked is None:
            return
        self.bitDialog = BitSelectionDialog(self, self.ui.spinBoxEndMask,
                                            self.ui.spinBoxEndId)
        self.bitDialog.show()

    def accept(self):
        average_group = str(self.ui.comboBoxAvgGroup.currentText())
        start_use_events = self.ui.radioButtonStartUseEvents.isChecked()
        end_use_events = self.ui.radioButtonEndUseEvents.isChecked()
        start_use_start = self.ui.radioButtonStartUseStart.isChecked()
        end_use_start = self.ui.radioButtonEndUseStart.isChecked()
        start_id = self.ui.spinBoxStartId.value()
        start_mask = self.ui.spinBoxStartMask.value()
        end_id = self.ui.spinBoxEndId.value()
        end_mask = self.ui.spinBoxEndMask.value()
        start_offset = self.ui.doubleSpinBoxStartOffset.value()
        end_offset = self.ui.doubleSpinBoxEndOffset.value()

        if start_use_events:
            start = ('events', start_id, start_mask, start_offset)
        elif start_use_start:
            start = ('start', None, None, start_offset)
        else:
            start = ('end', None, None, start_offset)

        if end_use_events:
            end = ('events', end_id, end_mask, end_offset)
        elif end_use_start:
            end = ('start', None, None, end_offset)
        else:
            end = ('end', None, None, end_offset)

        interval = (average_group, start, end)

        self.parent.add_intervals([('dynamic', interval)])
        self.close()



import os
import shutil
import tempfile
import mne

from meggie.mainwindow.preferences import PreferencesHandler

from meggie.experiment import initialize_new_experiment
from meggie.experiment import open_existing_experiment

def test_experiment_and_subject():
    with tempfile.TemporaryDirectory() as dirpath:
        sample_folder = mne.datasets.sample.data_path()
        sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

        # Create preferences object to store working directory
        # Also set prefs_path inside tempdir
        prefs = PreferencesHandler()
        prefs.workspace = dirpath
        prefs.prefs_path = os.path.join(dirpath, '.meggieprefs')

        # create experiment (creates experiment directory inside working directory,
        # also saves prefs (previous_experiment is set) and saves .exp file)
        name = 'Test experiment äö€ê*ë'
        author = 'Author'
        experiment = initialize_new_experiment(name, author, prefs)

        # add two subjects based on sample_audvis_raw
        # (makes copy of the file and adds it into subject directory inside experiment directory,
        # also creates and adds subject objects to the experiment object)
        experiment.create_subject('subject_1', sample_fname)
        experiment.create_subject('subject_2', sample_fname)

        # save experiment
        experiment.save_experiment_settings()

        loaded_experiment = open_existing_experiment(prefs, 
            path=experiment.path)

        assert(experiment.name == loaded_experiment.name)
        assert(set(experiment.subjects.keys()) == set(loaded_experiment.subjects.keys()))
        assert(experiment.subjects['subject_1'].name == loaded_experiment.subjects['subject_1'].name)




""" Contains a class for logic of about dialog.
"""

from PyQt5 import QtCore
from PyQt5 import QtWidgets

from meggie.mainwindow.dialogs.aboutDialogUi import Ui_Dialog


class AboutDialog(QtWidgets.QDialog):
    """ Contains the main logic for about dialog.
    """

    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        version = ''
        try:
            import pkg_resources
            version = pkg_resources.get_distribution("meggie").version
        except BaseException:
            pass

        self.ui.lineEditVersion.setText(str(version))

    def on_pushButtonClose_clicked(self, checked=None):
        if checked is None:
            return
        self.close()



""" UI layout for simple dialog
"""
from PyQt5 import QtCore
from PyQt5 import QtGui
from PyQt5 import QtWidgets

class Ui_SimpleDialog(object):
    """ Contains layout of very simple reusable dialog
    """
    def setupUi(self, dialog):
        dialog.setObjectName("CreateEvokedDialog")
        dialog.resize(364, 530)

        self.gridLayout = QtWidgets.QGridLayout(dialog)
        self.gridLayout.setObjectName("gridLayout")

        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)

        self.horizontalLayoutButtons = QtWidgets.QHBoxLayout()
        self.horizontalLayoutButtons.setObjectName("horizontalLayoutButtons")
        self.horizontalLayoutButtons.addItem(spacerItem)

        self.pushButtonCancel = QtWidgets.QPushButton(dialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayoutButtons.addWidget(self.pushButtonCancel)

        self.pushButtonBatch = QtWidgets.QPushButton(dialog)
        self.pushButtonBatch.setObjectName("pushButtonBatch")
        self.horizontalLayoutButtons.addWidget(self.pushButtonBatch)

        self.pushButtonApply = QtWidgets.QPushButton(dialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayoutButtons.addWidget(self.pushButtonApply)

        self.gridLayout.addLayout(self.horizontalLayoutButtons, 2, 0, 1, 1)

        self.scrollArea = QtWidgets.QScrollArea(dialog)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")

        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 344, 479))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")

        self.gridLayoutScrollArea = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayoutScrollArea.setObjectName("gridLayoutScrollArea")

        self.groupBoxBatching = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBatching.setObjectName("groupBoxBatching")
        self.gridLayoutScrollArea.addWidget(self.groupBoxBatching, 1, 0, 1, 1)

        self.gridLayoutBatching = QtWidgets.QGridLayout(self.groupBoxBatching)
        self.gridLayoutBatching.setContentsMargins(9, 9, 9, 9)
        self.gridLayoutBatching.setObjectName("gridLayoutBatching")

        self.batchingWidgetPlaceholder = QtWidgets.QWidget(self.groupBoxBatching)
        self.batchingWidgetPlaceholder.setMinimumSize(QtCore.QSize(300, 300))
        self.batchingWidgetPlaceholder.setObjectName("batchingWidgetPlaceholder")
        self.gridLayoutBatching.addWidget(self.batchingWidgetPlaceholder, 0, 0, 1, 1)

        self.groupBoxInfo = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxInfo.setObjectName("groupBoxInfo")

        self.formLayout = QtWidgets.QFormLayout(self.groupBoxInfo)
        self.formLayout.setObjectName("formLayout")

        self.labelName = QtWidgets.QLabel(self.groupBoxInfo)
        self.labelName.setObjectName("labelName")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelName)

        self.lineEditName = QtWidgets.QLineEdit(self.groupBoxInfo)
        self.lineEditName.setObjectName("lineEditName")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.lineEditName)

        self.gridLayoutScrollArea.addWidget(self.groupBoxInfo, 0, 0, 1, 1)

        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayoutScrollArea.addItem(spacerItem, 2, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(dialog)

        self.pushButtonCancel.clicked.connect(dialog.close)
        self.pushButtonApply.clicked.connect(dialog.accept)
        self.pushButtonBatch.clicked.connect(dialog.acceptBatch)
        QtCore.QMetaObject.connectSlotsByName(dialog)

    def retranslateUi(self, dialog):
        _translate = QtCore.QCoreApplication.translate
        dialog.setWindowTitle(_translate("SimpleDialog", "Meggie - Simple dialog"))
        self.pushButtonCancel.setText(_translate("SimpleDialog", "Cancel"))
        self.pushButtonBatch.setText(_translate("SimpleDialog", "Batch"))
        self.pushButtonApply.setText(_translate("SimpleDialog", "Apply"))
        self.groupBoxBatching.setTitle(_translate("SimpleDialog", "Batching"))
        self.groupBoxInfo.setTitle(_translate("SimpleDialog", "Info"))
        self.labelName.setText(_translate("SimpleDialog", "Name:"))




""" Contains implementation for raw measurement info
"""
from meggie.mainwindow.dynamic import InfoAction

from meggie.utilities.measurement_info import MeasurementInfo
from meggie.utilities.channels import is_montage_set


class Info(InfoAction):
    """ Shows measurement information on a info box
    """

    def run(self):

        message = ""
        try:
            subject = self.experiment.active_subject
            raw = subject.get_raw()
            mi = MeasurementInfo(raw)

            maxfilter_applied = subject.sss_applied
            ica_applied = subject.ica_applied
            rereferenced = subject.rereferenced

            try:
                eeg_montage_set = is_montage_set(raw.info, 'eeg')
            except Exception as exc:
                eeg_montage_set = False

            message += "Subject name: {0}\n".format(mi.subject_name)
            message += "Date: {0}\n".format(mi.date)
            message += "Length: {0:.2f} s\n".format(raw.times[-1])
            message += "Highpass: {0} Hz\n".format(mi.high_pass)
            message += "Lowpass: {0} Hz\n".format(mi.low_pass)
            message += "Sampling rate: {0} Hz\n".format(mi.sampling_freq)
            message += "Bads: " + ', '.join(raw.info['bads']) + '\n'

            message += "Maxfilter applied: {0}\n".format(str(maxfilter_applied))
            message += "ICA applied: {0}\n".format(str(ica_applied))
            message += "Rereferenced: {0}\n".format(rereferenced)
            message += "EEG montage set: {0}\n".format(eeg_montage_set)
        except Exception as exc:
            return ""

        return message



""" Contains a class for logic of the bit selection dialog.
"""
from PyQt5 import QtWidgets
from PyQt5.Qt import QPushButton, pyqtSlot

from meggie.utilities.dialogs.bitSelectionDialogUi import Ui_Dialog


class BitSelectionDialog(QtWidgets.QDialog):
    """ Contains the logic for bit selection dialog.
    """

    def __init__(self, parent, target_mask, target_id):
        QtWidgets.QDialog.__init__(self)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.intervals = []
        self.button_count = 16

        self.parent = parent
        self.target_mask = target_mask
        self.target_id = target_id

        self.ui.labelID.setText('0')
        # 2^0 + 2^1 + ... + 2^(button_count - 1)
        self.ui.labelMask.setText(str(pow(2, self.button_count) - 1))

        for i in range(1, self.button_count + 1):
            getattr(self.ui, "pushButton" + str(i)).setText(" ")
            (lambda x: getattr(self.ui, "pushButton" + str(x)).clicked.connect(
                lambda: self._bit_clicked(getattr(self.ui,
                                                 "pushButton" + str(x)))))(i)

    @pyqtSlot(QPushButton)
    def _bit_clicked(self, button):
        id_operation, mask_operation = None, None

        if button.text() == ' ':
            button.setText('0')
            mask_operation = '-'
        elif button.text() == '0':
            button.setText('1')
            id_operation = '+'
        elif button.text() == '1':
            button.setText(' ')
            mask_operation = '+'
            id_operation = '-'

        current_id = int(self.ui.labelID.text())
        current_mask = int(self.ui.labelMask.text())
        current_button_number = int(button.objectName()[len("pushButton"):])

        new_id, new_mask = current_id, current_mask

        if id_operation == '+':
            new_id = current_id + pow(2, current_button_number - 1)
        elif id_operation == '-':
            new_id = current_id - pow(2, current_button_number - 1)

        if mask_operation == '+':
            new_mask = current_mask + pow(2, current_button_number - 1)
        elif mask_operation == '-':
            new_mask = current_mask - pow(2, current_button_number - 1)

        self.ui.labelID.setText(str(new_id))
        self.ui.labelMask.setText(str(new_mask))

    def accept(self):
        self.target_id.setValue(int(self.ui.labelID.text()))
        self.target_mask.setValue(int(self.ui.labelMask.text()))
        self.close()



""" Contains controlling logic for the tfr implementation
"""

import numpy as np
import matplotlib.pyplot as plt
import mne

from meggie.utilities.plotting import color_cycle
from meggie.utilities.plotting import create_channel_average_plot
from meggie.utilities.channels import average_to_channel_groups
from meggie.utilities.channels import iterate_topography
from meggie.utilities.units import get_power_unit


def _compute_tse(meggie_tfr, fmin, fmax):
    tfrs = meggie_tfr.content

    fmin_idx = np.where(meggie_tfr.freqs >= fmin)[0][0]
    fmax_idx = np.where(meggie_tfr.freqs <= fmax)[0][-1]

    if fmax_idx <= fmin_idx:
        raise Exception('Something wrong with the frequencies')

    tses = {}
    for key, tfr in tfrs.items():
        tse = np.mean(tfr.data[:, fmin_idx:fmax_idx+1, :], axis=1)
        tses[key] = tse

    return tses


def _crop_and_correct_to_baseline(tse, blmode, blstart, blend, tmin, tmax, times):
    tmin_idx = np.where(times >= tmin)[0][0]
    tmax_idx = np.where(times <= tmax)[0][-1]

    if blmode:
        if blstart < tmin:
            raise Exception(
                'Baseline start should not be earlier than crop start.')

        if blend > tmax:
            raise Exception(
                'Baseline end should not be later than crop end.')

        # correct to baseline
        tse = mne.baseline.rescale(tse, times, baseline=(blstart, blend), 
                                   mode=blmode)

        # crop
        tse = tse[:, tmin_idx:tmax_idx+1]

    return times[tmin_idx:tmax_idx+1], tse


def plot_tse_topo(subject, tfr_name, blmode, blstart, blend, 
                  tmin, tmax, fmin, fmax, ch_type):
    """ Plots a tse topography.
    """
    meggie_tfr = subject.tfr.get(tfr_name)

    tses = _compute_tse(meggie_tfr, fmin, fmax)

    info = meggie_tfr.info
    if ch_type == 'meg':
        picked_channels = [ch_name for ch_idx, ch_name in enumerate(info['ch_names'])
                            if ch_idx in mne.pick_types(info, meg=True, eeg=False)]
    else:
        picked_channels = [ch_name for ch_idx, ch_name in enumerate(info['ch_names'])
                            if ch_idx in mne.pick_types(info, meg=False, eeg=True)]
    info = info.copy().pick_channels(picked_channels)

    ch_names = meggie_tfr.ch_names
    colors = color_cycle(len(tses))

    def individual_plot(ax, info_idx, names_idx):
        """
        """
        ch_name = ch_names[names_idx]

        title = ' '.join([tfr_name, ch_name])
        ax.figure.canvas.set_window_title(title.replace(' ', '_'))
        ax.figure.suptitle(title)
        ax.set_title('')

        for color_idx, (key, tse) in enumerate(sorted(tses.items())):
            times, tse = _crop_and_correct_to_baseline(
                tse, blmode, blstart, blend, tmin, tmax, meggie_tfr.times)
            ax.plot(times, tse[names_idx], color=colors[color_idx], label=key)
            ax.axhline(0, color='black')
            ax.axvline(0, color='black')

        ax.legend()

        ax.set_xlabel('Time (s)')
        ax.set_ylabel('Power ({})'.format(get_power_unit(
            mne.io.pick.channel_type(info, info_idx),
            False
        )))

        plt.show()

    fig = plt.figure()
    for ax, info_idx, names_idx in iterate_topography(
            fig, info, ch_names, individual_plot):

        handles = []
        for color_idx, (key, tse) in enumerate(sorted(tses.items())):
            times, tse = _crop_and_correct_to_baseline(
                tse, blmode, blstart, blend, tmin, tmax, meggie_tfr.times)
            handles.append(ax.plot(times, tse[names_idx], linewidth=0.2, 
                           color=colors[color_idx],
                           label=key)[0])

    fig.legend(handles=handles)
    title = '{0}_{1}'.format(tfr_name, ch_type)
    fig.canvas.set_window_title(title)
    plt.show()


def plot_tse_averages(subject, tfr_name, blmode, blstart, blend,
                      tmin, tmax, fmin, fmax, channel_groups):
    """ Plots tse averages.
    """
    meggie_tfr = subject.tfr.get(tfr_name)

    tses = _compute_tse(meggie_tfr, fmin, fmax)

    ch_names = meggie_tfr.ch_names
    info = meggie_tfr.info
    colors = color_cycle(len(tses))
    conditions = meggie_tfr.content.keys()

    averages = {}
    for key, tse in sorted(tses.items()):
        data_labels, averaged_data = average_to_channel_groups(
            tse, info, ch_names, channel_groups)

        times, averaged_data = _crop_and_correct_to_baseline(
            averaged_data, blmode, blstart, blend, tmin, tmax, meggie_tfr.times)

        for label_idx, label in enumerate(data_labels):
            if not label in averages:
                averages[label] = []
            averages[label].append((key, times, averaged_data[label_idx]))

    ch_types = sorted(set([label[0] for label in averages.keys()]))
    for ch_type in ch_types:

        ch_groups = sorted([label[1] for label in averages.keys()
                            if label[0] == ch_type])

        def plot_fun(ax_idx, ax):
            ch_group = ch_groups[ax_idx]
            ax.set_title(ch_group)
            ax.set_xlabel('Time (s)')
            ax.set_ylabel('Power ({})'.format(get_power_unit(
                ch_type, False)))

            for color_idx, (key, times, curve) in enumerate(averages[(ch_type, ch_group)]):
                ax.plot(times, curve, color=colors[color_idx], label=key)

            ax.axhline(0, color='black')
            ax.axvline(0, color='black')

        title = ' '.join([tfr_name, ch_type])
        legend = list(zip(conditions, colors))
        create_channel_average_plot(len(ch_groups), plot_fun, title, 
                                    legend)

    plt.show()




""" Contains useful functions for handling events.
"""

import logging

import numpy as np
import mne

from collections import OrderedDict


def _should_take(id_, mask, event):
    """Check if event has same non-masked bits as id_.
    """
    id_bin = '{0:016b}'.format(id_)
    mask_bin = '{0:016b}'.format(mask)
    event_bin = '{0:016b}'.format(event[2])

    take_event = True
    for i in range(len(mask_bin)):
        if int(mask_bin[i]) == 1:
            continue
        if int(id_bin[i]) != int(event_bin[i]):
            take_event = False
            break

    return take_event


def find_events(raw, stim_ch=None, mask=0, id_=None):
    """Finds events using mne.find_events, but implements
    own masking procedure.

    The masking procedure is simple. Event is ok if it has the same
    bits as the id everywhere where the mask is zero. Thus mask 
    specifies bits that are not cared about.

    Parameters
    ----------
    raw : mne.io.Raw
        The raw object.
    stim_ch : str
        Name of the stimulus channel.
    mask : int
        The mask as integer, is converted to binary representation.
    id_ : int
        The id as integer, is converted to binary representation.

    Returns
    -------
    np.array
        MNE-style events array, i.e. np.array of shape (n_events, 3).

    """

    events = mne.find_events(raw, stim_channel=stim_ch, shortest_event=1,
                             uint_cast=True)

    if mask or id_:
        events = list(filter(
            lambda event: _should_take(id_, mask, event), events))
        events = np.array(events)

    # remove spurious events (only one sample difference to next event)
    counter = 0
    for idx in reversed(range(1, len(events))):
        if events[idx][0] - events[idx - 1][0] < 2:
            events = np.delete(events, idx - 1, axis=0)
            counter += 1

    if counter > 0:
        message = (str(counter) +
                   " events dropped because they seem spurious "
                   "(only one sample difference to next event)")
        logging.getLogger('ui_logger').warning(message)

    return events


def find_stim_channel(raw):
    """ Finds the appropriate stim channel from raw.

    Heuristically just looks for STI101 or STI014.

    Parameters
    ----------
    raw : mne.io.Raw
        The raw object

    Returns
    -------
    str
        Channel name of the stimulus channel.
    """
    channels = raw.info.get('ch_names')
    if 'STI101' in channels:
        return 'STI101'
    elif 'STI 101' in channels:
        return 'STI 101'
    elif 'STI 014' in channels:
        return 'STI 014'
    elif 'STI014' in channels:
        return 'STI014'


def update_stim_channel(raw, events):
    """Writes events to the stimulus channel.

    If no stimulus channel, create one, and then write.

    Parameters
    ----------
    raw : mne.io.Raw
        The raw object.
    events : np.array
        MNE-style events array.
    """
    # time on in samples
    length = 5

    stim_channel = find_stim_channel(raw)
    
    if not stim_channel:
        # create stim_channel
        info = mne.create_info(['STI101'], raw.info['sfreq'], ['stim'])
        stim_raw = mne.io.RawArray(np.zeros((1, len(raw.times))), info)
        raw.add_channels([stim_raw], force_update_info=True)
        stim_channel = 'STI101'

    ch_idx = raw.info['ch_names'].index(stim_channel)
    for event in events:
        start = event[0] - raw.first_samp
        raw._data[ch_idx][start:start+length] = event[2]


def events_from_annotations(subject, conversion_info):
    """Convert annotations to events and merge them with the stim channel.

    Parameters
    ----------
    subject : meggie.subject.Subject
        The subject to do the conversion.
    conversion_info : list
        List of tuples like (annotation_name, event_id, use_start),
        where use_start is a boolean and False means end of the annotation.
    """

    raw = subject.get_raw()

    events = []
    for annotation_name, event_id, use_start in conversion_info:
        for idx, annot_desc in enumerate(raw.annotations.description):
            if annot_desc != annotation_name:
                continue

            if use_start:
                onset_time = raw.annotations.onset[idx]
            else:
                onset_time = (raw.annotations.onset[idx] + 
                              raw.annotations.duration[idx])

            tidx = int(onset_time*raw.info['sfreq'])

            events.append([tidx, 0, event_id])

    events = np.array(sorted(events, key=lambda x: x[0]))

    update_stim_channel(raw, events)


def _find_event_times(raw, event_id, mask):
    """Given the event_id and mask, find the event times.
    """
    stim_ch = find_stim_channel(raw)
    sfreq = raw.info['sfreq']

    events = find_events(raw, stim_ch, mask, event_id)
    times = [(event[0] - raw.first_samp) / sfreq for event in events]
    return times


def get_raw_blocks_from_intervals(subject, intervals):
    """ Creates RawAarrays from time interval specifications.

    Parameters
    ----------
    subject : meggie.subject.Subject
        The subject whose raw data is used.
    intervals : list
        List of time interval specifications like (type, (group, tmin, tmax)),
        where type can be 'dynamic' or 'fixed' and group is a str.

    Returns
    -------
    OrderedDict
        Times of the intervals organized by average groups.
    OrderedDict
        RawArrays organized by average groups.

    """
    raw = subject.get_raw()

    raw_times = raw.times.copy()

    raw_blocks = OrderedDict()
    times = OrderedDict()
    for ival_type, (avg_group, start, end) in intervals:
        if avg_group not in raw_blocks:
            raw_blocks[avg_group] = []
            times[avg_group] = []

        if ival_type == 'fixed':
            block = raw.copy().crop(tmin=start, tmax=end)
            raw_blocks[avg_group].append(block)
            times[avg_group].append((start, end))
        else:
            # the following code finds all start points of intervals by events or
            # start of recording. then matching end point is found by
            # (can be same) other events or end of recording.
            if start[0] == 'events':
                start_times = _find_event_times(raw, start[1], start[2])
            elif start[0] == 'start':
                start_times = [raw_times[0]]
            elif start[0] == 'end':
                start_times = [raw_times[-1]]

            for start_time in start_times:
                if end[0] == 'events':
                    end_times = _find_event_times(raw, end[1], end[2])
                    found = False
                    for end_time in end_times:
                        # use equality so that one can also specify same trigger for
                        # start and end (with different offsets)

                        # sanity check
                        if np.isclose((end_time + end[3]) - (start_time + start[3]), 0):
                            continue

                        if end_time >= start_time:
                            found = True
                            break
                    if not found:
                        logging.getLogger('ui_logger').info(
                            'Found start event with no matching end event')
                        continue
                elif end[0] == 'start':
                    end_time = raw_times[0]
                elif end[0] == 'end':
                    end_time = raw_times[-1]

                # crop with offsets
                times[avg_group].append((start_time + start[3],
                                         end_time + end[3]))
                block = raw.copy().crop(tmin=(start_time + start[3]),
                                        tmax=(end_time + end[3]))
                raw_blocks[avg_group].append(block)

    for key in raw_blocks:
        if len(raw_blocks[key]) == 0:
            raise Exception('Was not able to find raw segments for all groups')
     
    return times, raw_blocks




"""Contains useful functions for developers.
"""

def debug_trace():
    """Helper to allow debugging with pyqt application.
    
    The standard way of using "import pdb; pdb.set_trace()"
    does not work with Qt application as the Qt fills the terminal
    with messages. Using this bypasses it:
    from meggie.utilities.debug import debug_trace; debug_trace()
    When stopped, use "u" to go to parent function.

    After debugging should call PyQt4.QtCore.pyqtRestoreInputHook, though
    often one just quits and restarts.
    """
    from PyQt5.QtCore import pyqtRemoveInputHook
    pyqtRemoveInputHook()

    import pdb
    pdb.set_trace()



"""Contains a class for handling reading and storing of 
global preferences such as the previous experiment and 
the tab settings. Normally saved to ~/.meggieprefs.
"""

import os
import configparser
import logging

from meggie.utilities.filemanager import homepath


class PreferencesHandler(object):
    """ Class for storing and setting preferences.
    """

    def __init__(self):
        self.prefs_path = ""
        self.workspace = ""
        self.previous_experiment_name = ""
        self.auto_load_last_open_experiment = False
        self.active_plugins = []

        self.read_preferences_from_disk()

    def write_preferences_to_disk(self):
        """ Writes the preferences to file system, in INI style.
        """
        config = configparser.RawConfigParser()
        config.add_section('MiscOptions')
        config.add_section('Workspace')

        # Sanity of these values is assumed to be checked by the calling method

        config.set('Workspace', 'workspaceDir', self.workspace)
        logging.getLogger('ui_logger').info("Workspace: " + 
                                            self.workspace)

        config.set('MiscOptions', 'previousExperimentName',
                   self.previous_experiment_name)
        logging.getLogger('ui_logger').info("Previous experiment: " + 
                                            self.previous_experiment_name)

        if self.auto_load_last_open_experiment:
            config.set('MiscOptions', 'autoReloadPreviousExperiment', 'True')
        else:
            config.set('MiscOptions', 'autoReloadPreviousExperiment', 'False')
        logging.getLogger('ui_logger').info("Auto reload previous experiment: " + 
                                            str(self.auto_load_last_open_experiment))

        config.set('MiscOptions', 'activePlugins', ','.join(self.active_plugins))
        logging.getLogger('ui_logger').info("Active plugins: " + str(self.active_plugins))

        path = self.prefs_path
        if not path:
            path = os.path.join(homepath(), '.meggieprefs')

        with open(path, 'w') as configfile:
            config.write(configfile)

    def read_preferences_from_disk(self):
        """ Reads the preferences from file system into attributes.
        """
        filename = os.path.join(homepath(), '.meggieprefs')
        if os.path.isfile(filename):
            config = configparser.RawConfigParser()
            config.read(filename)

        try:
            self.workspace = config.get('Workspace', 'workspaceDir')
        except Exception as exc:
            pass

        try:
            auto_reload = config.get('MiscOptions', 
                                     'autoReloadPreviousExperiment')
            if auto_reload == "True":
                self.auto_load_last_open_experiment = True
        except Exception as exc:
            pass

        try:
            self.previous_experiment_name = config.get(
                'MiscOptions', 'previousExperimentName')
        except Exception as exc:
            pass

        try:
            active_plugins = config.get('MiscOptions', 'activePlugins')
            self.active_plugins = active_plugins.split(',')
        except Exception as exc:
            pass




""" Contains a class for logic of ica dialog.
"""

import logging

from PyQt5 import QtWidgets

from meggie.actions.raw_ica.dialogs.icaDialogUi import Ui_Dialog

from meggie.actions.raw_ica.controller.ica import plot_topographies
from meggie.actions.raw_ica.controller.ica import plot_sources
from meggie.actions.raw_ica.controller.ica import plot_properties
from meggie.actions.raw_ica.controller.ica import plot_changes
from meggie.actions.raw_ica.controller.ica import compute_ica

from meggie.utilities.threading import threaded
from meggie.utilities.messaging import exc_messagebox


class ICADialog(QtWidgets.QDialog):
    """ Contains logic for ICA dialog.
    """

    def __init__(self, parent, experiment, random_state, on_apply=None):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)

        self.parent = parent
        self.experiment = experiment
        self.on_apply = on_apply
        self.random_state = random_state

        # change normal list widgets to multiselect widgets
        self.ui.listWidgetNotRemoved.setSelectionMode(
            QtWidgets.QAbstractItemView.ExtendedSelection)
        self.ui.listWidgetRemoved.setSelectionMode(
            QtWidgets.QAbstractItemView.ExtendedSelection)

        self._reset()

    def _reset(self):
        self.ica = None
        self.ui.listWidgetNotRemoved.clear()
        self.ui.listWidgetRemoved.clear()
        self.not_removed = []
        self.removed = []
        self.component_info = {}
        self.params = {}

    def on_pushButtonCompute_clicked(self, checked=None):
        if checked is None:
            return

        # start by clearing out the previous things
        self._reset()

        n_components = self.ui.doubleSpinBoxNComponents.value()
        method = 'fastica'
        max_iter = self.ui.spinBoxMaxIter.value()

        raw = self.experiment.active_subject.get_raw()

        @threaded
        def _compute_ica():
            return compute_ica(raw, n_components, method, max_iter, self.random_state)

        try:
            self.ica = _compute_ica(do_meanwhile=self.parent.update_ui)
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        self.params['n_components'] = n_components
        self.params['method'] = method
        self.params['max_iter'] = max_iter
        self.params['random_state'] = self.random_state

        for idx in range(self.ica.n_components_):
            label = 'Component ' + str(idx)
            self.ui.listWidgetNotRemoved.addItem(label)
            self.component_info[label] = idx
            self.not_removed.append(label)

        logging.getLogger('ui_logger').info('Computing ICA model finished.')

    def on_pushButtonTransfer_clicked(self, checked=None):
        if checked is None:
            return

        # gather contents of the widgets
        not_removed_selected = [item.text() for item in
                                self.ui.listWidgetNotRemoved.selectedItems()]
        removed_selected = [item.text() for item in
                            self.ui.listWidgetRemoved.selectedItems()]

        # update the "backend"
        for item in not_removed_selected:
            self.not_removed.remove(item)
            self.removed.append(item)

        for item in removed_selected:
            self.removed.remove(item)
            self.not_removed.append(item)

        self.not_removed.sort(key=lambda x: self.component_info[x])
        self.removed.sort(key=lambda x: self.component_info[x])

        # clear ui
        self.ui.listWidgetNotRemoved.clear()
        self.ui.listWidgetRemoved.clear()

        # fill again
        for item in self.removed:
            self.ui.listWidgetRemoved.addItem(item)
        for item in self.not_removed:
            self.ui.listWidgetNotRemoved.addItem(item)

    def on_listWidgetNotRemoved_clicked(self):
        # Enforce only one list have a selected item
        widget = self.ui.listWidgetRemoved
        for i in range(widget.count()):
            widget.item(i).setSelected(False)

    def on_listWidgetRemoved_clicked(self):
        # Enforce only one list have a selected item
        widget = self.ui.listWidgetNotRemoved
        for i in range(widget.count()):
            widget.item(i).setSelected(False)

    def on_pushButtonPlotTopographies_clicked(self, checked=None):
        if checked is None:
            return

        raw = self.experiment.active_subject.get_raw()

        try:
            plot_topographies(self.ica, len(self.component_info))
        except Exception as exc:
            exc_messagebox(self, exc)
            return

    def on_pushButtonPlotSources_clicked(self, checked=None):
        if checked is None:
            return

        raw = self.experiment.active_subject.get_raw()

        try:
            plot_sources(raw, self.ica)
        except Exception as exc:
            exc_messagebox(self, exc)
            return

    def on_pushButtonPlotProperties_clicked(self, checked=None):
        if checked is None:
            return

        picks = self._get_picks()

        if not picks:
            return

        raw = self.experiment.active_subject.get_raw()

        try:
            plot_properties(raw, self.ica, picks)
        except Exception as exc:
            exc_messagebox(self, exc)
            return

    def on_pushButtonPlotChanges_clicked(self, checked=None):
        if checked is None:
            return

        raw = self.experiment.active_subject.get_raw()

        indices = [self.component_info[name] for name in self.removed]

        try:
            plot_changes(raw, self.ica, indices)
        except Exception as exc:
            exc_messagebox(self, exc)
            return

    def _get_picks(self):
        not_removed_selected = [item.text() for item in
                                self.ui.listWidgetNotRemoved.selectedItems()]
        removed_selected = [item.text() for item in
                            self.ui.listWidgetRemoved.selectedItems()]

        picks = [self.component_info[label]
                 for label in not_removed_selected]
        picks.extend([self.component_info[label]
                      for label in removed_selected])

        picks = sorted(picks)
        return picks

    def accept(self):
        if not self.ica:
            return

        raw = self.experiment.active_subject.get_raw()

        indices = [self.component_info[name] for name in self.removed]

        try:
            @threaded
            def apply_ica_wrapper():
                self.ica.apply(raw, exclude=indices)

            apply_ica_wrapper(do_meanwhile=self.parent.update_ui)

            self.experiment.active_subject.save()
            self.experiment.active_subject.ica_applied = True
            self.experiment.save_experiment_settings()
     
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        self.parent.initialize_ui()

        # for logging purposes, call the handler
        if self.on_apply:
            self.params['exclude'] = indices
            self.on_apply(self.experiment.active_subject, self.params)

        self._reset()
        self.close()



""" Contains a logic for setting active plugins.
"""

import logging
import pkg_resources
import json

from PyQt5 import QtWidgets

from meggie.mainwindow.dynamic import find_all_plugins
from meggie.mainwindow.dynamic import find_all_package_specs

from meggie.mainwindow.dialogs.activePluginsDialogUi import Ui_activePluginsDialog


class ActivePluginsDialog(QtWidgets.QDialog):
    """ Contains logic for active plugins dialog.
    """

    def __init__(self, active_plugins, parent, handler):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_activePluginsDialog()
        self.ui.setupUi(self)

        self.handler = handler

        plugins = find_all_plugins()
        package_specs = find_all_package_specs()


        plugin_info = []
        for source, package_spec in package_specs.items():
            if source not in plugins:
                continue

            id_ = package_spec['id']
            name = package_spec['name']
            author = package_spec['author']

            plugin_info.append((id_, name, author))
        self.plugin_info = plugin_info

        for id_, name, author in plugin_info:
            text = name + ' - ' + author
            item = QtWidgets.QListWidgetItem(text)
            self.ui.listWidgetPlugins.addItem(item)
            if id_ in active_plugins:
                item.setSelected(True)

    def accept(self):
        idxs = [elem.row() for elem in self.ui.listWidgetPlugins.selectedIndexes()]
        active_plugins = [self.plugin_info[idx][0] for idx in idxs]
        self.handler(active_plugins)
        self.close()



"""Provides functions to generate experiments from sample data.
"""

import os
import sys
import shutil
import tempfile

import mne
import numpy as np

from meggie.subject import Subject
from meggie.mainwindow.preferences import PreferencesHandler

from meggie.experiment import initialize_new_experiment
from meggie.experiment import open_existing_experiment


def create_experiment(experiment_folder, experiment_name, subjects_raw, overwrite=False):
    """Creates experiment from list of raw objects.

    Parameters
    ----------
    experiment_folder : str
        Where the experiment is stored.
    experiment_name : str
        Name of the experiment.
    subjects_raw : list
        List of raws, one for each subject.
    overwrite : bool
        Whether to overwrite previous experiment with a same name. 

    Returns
    -------
    meggie.experiment.Experiment
        An experiment object containing the subjects.
    """

    if os.path.exists(os.path.join(experiment_folder, experiment_name)):
        if overwrite:
            shutil.rmtree(os.path.join(experiment_folder, experiment_name))
        else:
            raise Exception('Experiment already exists')

    # Create preferences object to store working directory
    prefs = PreferencesHandler()
    prefs.workspace = experiment_folder

    # create experiment (creates experiment directory inside working directory)
    name = experiment_name
    author = 'Test'
    experiment = initialize_new_experiment(name, author, prefs, set_previous_experiment=False)

    for subject_idx, raw in enumerate(subjects_raw):
        with tempfile.TemporaryDirectory() as sample_folder:

            fname = 'sample_' + str(subject_idx+1).zfill(2) + '-raw.fif'

            raw_path = os.path.join(sample_folder, fname)
            raw.save(raw_path)

            subject_name = fname.split('.fif')[0]
            experiment.create_subject(subject_name, 
                                      raw_path)

    experiment.save_experiment_settings()
    return experiment


def create_evoked_conditions_experiment(experiment_folder, experiment_name, overwrite=False, n_subjects=35):
    """Generate multisubject experiment based on sample_audvis_raw for testing purposes.

    Subjects are generated so that each will contain different versions of two LA and RA auditory responses.

    Parameters
    ----------
    experiment_folder : str
        Where the experiment is stored.
    experiment_name : str
        Name of the experiment.
    overwrite : bool
        Whether to overwrite previous experiment with a same name. 
    n_subjects : int
        How many subjects to add.

    Returns
    -------
    meggie.experiment.Experiment
        An experiment object containing the subjects.
    """
    sample_folder = mne.datasets.sample.data_path()
    sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

    subjects_raw = []

    # generate data
    raw = mne.io.read_raw_fif(sample_fname, preload=True)
    events = mne.find_events(raw)

    la_events = [ev for ev in events if ev[2] == 1]
    ra_events = [ev for ev in events if ev[2] == 2]

    for subject_idx in range(n_subjects):
        data = []
        combined_events = la_events[subject_idx*2:subject_idx*2+2]
        combined_events.extend(ra_events[subject_idx*2:subject_idx*2+2])

        if len(combined_events) != 4:
            raise Exception('Something wrong with event counts')

        data = []
        for event in combined_events:
            tidx = event[0]
            try:
                ev_before = [ev for ev in events if ev[0] < tidx][-1]
                tmin = (ev_before[0] - raw.first_samp + 20) / raw.info['sfreq']
            except:
                tmin=0.0

            try:
                ev_after = [ev for ev in events if ev[0] > tidx][0]
                tmax = (ev_after[0] - raw.first_samp - 20) / raw.info['sfreq']
            except:
                tmax=None

            data.append(raw.copy().crop(tmin,tmax)._data)

        subject_raw = mne.io.RawArray(
            np.concatenate(data, axis=1),
            raw.info,
            first_samp=0)

        subjects_raw.append(subject_raw)

    return create_experiment(experiment_folder, experiment_name, subjects_raw, overwrite=overwrite)


# allow creating experiment from the command line
if __name__ == '__main__':
    type_, experiment_folder, experiment_name = sys.argv[1:]

    if type_ == 'evoked_conditions':
        create_evoked_conditions_experiment(experiment_folder, experiment_name)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'batchingWidgetUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_BatchingWidget(object):
    def setupUi(self, BatchingWidget):
        BatchingWidget.setObjectName("BatchingWidget")
        BatchingWidget.resize(291, 276)
        self.layoutWidget = QtWidgets.QWidget(BatchingWidget)
        self.layoutWidget.setGeometry(QtCore.QRect(0, 0, 289, 31))
        self.layoutWidget.setObjectName("layoutWidget")
        self.headingLayout = QtWidgets.QHBoxLayout(self.layoutWidget)
        self.headingLayout.setContentsMargins(0, 0, 0, 0)
        self.headingLayout.setObjectName("headingLayout")
        self.checkBoxBatch = QtWidgets.QCheckBox(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkBoxBatch.sizePolicy().hasHeightForWidth())
        self.checkBoxBatch.setSizePolicy(sizePolicy)
        self.checkBoxBatch.setObjectName("checkBoxBatch")
        self.headingLayout.addWidget(self.checkBoxBatch)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.headingLayout.addItem(spacerItem)
        self.functionalityWidget = QtWidgets.QWidget(BatchingWidget)
        self.functionalityWidget.setGeometry(QtCore.QRect(0, 30, 289, 241))
        self.functionalityWidget.setObjectName("functionalityWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.functionalityWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.functionalityLayout = QtWidgets.QGridLayout()
        self.functionalityLayout.setObjectName("functionalityLayout")
        self.applyButtonsLayout = QtWidgets.QVBoxLayout()
        self.applyButtonsLayout.setObjectName("applyButtonsLayout")
        self.horizontalLayout_31 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_31.setObjectName("horizontalLayout_31")
        self.pushButtonApplyAll = QtWidgets.QPushButton(self.functionalityWidget)
        self.pushButtonApplyAll.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButtonApplyAll.sizePolicy().hasHeightForWidth())
        self.pushButtonApplyAll.setSizePolicy(sizePolicy)
        self.pushButtonApplyAll.setMinimumSize(QtCore.QSize(90, 0))
        self.pushButtonApplyAll.setMaximumSize(QtCore.QSize(90, 16777215))
        self.pushButtonApplyAll.setObjectName("pushButtonApplyAll")
        self.horizontalLayout_31.addWidget(self.pushButtonApplyAll)
        self.label_7 = QtWidgets.QLabel(self.functionalityWidget)
        self.label_7.setEnabled(True)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_31.addWidget(self.label_7)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_31.addItem(spacerItem1)
        self.applyButtonsLayout.addLayout(self.horizontalLayout_31)
        self.functionalityLayout.addLayout(self.applyButtonsLayout, 1, 0, 1, 1)
        self.subjectListLayout = QtWidgets.QVBoxLayout()
        self.subjectListLayout.setObjectName("subjectListLayout")
        self.labelSubjects = QtWidgets.QLabel(self.functionalityWidget)
        self.labelSubjects.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelSubjects.sizePolicy().hasHeightForWidth())
        self.labelSubjects.setSizePolicy(sizePolicy)
        self.labelSubjects.setObjectName("labelSubjects")
        self.subjectListLayout.addWidget(self.labelSubjects)
        self.listWidgetSubjects = QtWidgets.QListWidget(self.functionalityWidget)
        self.listWidgetSubjects.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidgetSubjects.sizePolicy().hasHeightForWidth())
        self.listWidgetSubjects.setSizePolicy(sizePolicy)
        self.listWidgetSubjects.setObjectName("listWidgetSubjects")
        self.subjectListLayout.addWidget(self.listWidgetSubjects)
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.subjectListLayout.addLayout(self.horizontalLayout_17)
        self.functionalityLayout.addLayout(self.subjectListLayout, 0, 0, 1, 1)
        self.verticalLayout.addLayout(self.functionalityLayout)

        self.retranslateUi(BatchingWidget)
        self.checkBoxBatch.toggled['bool'].connect(BatchingWidget.showWidget)
        QtCore.QMetaObject.connectSlotsByName(BatchingWidget)

    def retranslateUi(self, BatchingWidget):
        _translate = QtCore.QCoreApplication.translate
        BatchingWidget.setWindowTitle(_translate("BatchingWidget", "Meggie - Batch processing"))
        self.checkBoxBatch.setText(_translate("BatchingWidget", "Batch processing"))
        self.pushButtonApplyAll.setText(_translate("BatchingWidget", "Select all"))
        self.label_7.setText(_translate("BatchingWidget", "subjects on the list"))
        self.labelSubjects.setText(_translate("BatchingWidget", "Select subjects to include in batch"))




"""Contains functions that help with channel-related tasks.
"""

import scipy
import numpy as np
import mne

from mne.channels import _divide_to_regions


def is_montage_set(info, ch_type):
    """ Checks whether channel locations are found in the info for given channel type.

    Parameters
    ----------
    info : mne.Info
        Info structure containing channel information.
    ch_type : str
        Should be 'meg' or 'eeg'.

    Returns
    -------
    bool
        Whether the channel locations were found.
    """
    if ch_type == 'meg':
        picks = mne.pick_types(info, meg=True, eeg=False)
    else:
        picks = mne.pick_types(info, meg=False, eeg=True)

    ch_names = [ch_name for idx, ch_name in enumerate(info['ch_names'])
                if idx in picks]

    if not ch_names:
        raise Exception('Data does not contain channels of type ' + str(ch_type))

    info_filt = info.copy().pick_channels(ch_names)

    # check if there is no montage set..
    ch_norms = []
    for ch in info_filt['chs']:
        ch_norms.append(np.linalg.norm(ch['loc']))
    if np.all(np.isclose(ch_norms, ch_norms[0])):
        return False

    return True

def get_default_channel_groups(info, ch_type):
    """Returns channels grouped by standard locations
    (Left-frontal, Right-occipital, etc.). Grouping is done via
    geometric division from mne, to have a generic ability
    to divide different eeg caps into groups.

    Parameters
    ----------
    info : mne.Info
        Info structure containing the channel information
    ch_type : str
        Should be 'meg' or 'eeg'

    Returns
    -------
    dict
        A dictionary with groups (Left-temporal, etc.) as keys and 
        lists of channels as values.

    """

    if ch_type == 'meg':
        picks = mne.pick_types(info, meg=True, eeg=False)
    else:
        picks = mne.pick_types(info, meg=False, eeg=True)

    ch_names = [ch_name for idx, ch_name in enumerate(info['ch_names'])
                if idx in picks]
    if not ch_names:
        return {}

    info_filt = info.copy().pick_channels(ch_names)

    # check if there is no montage set..
    if not is_montage_set(info, ch_type):
        return {}

    regions = _divide_to_regions(info_filt, add_stim=False)

    ch_groups = {}
    for region_key, region in regions.items():
        region_ch_names = [info_filt['ch_names'][ch_idx] for ch_idx 
                           in region]
        ch_groups[region_key] = region_ch_names

    return ch_groups


def get_channels_by_type(info):
    """Returns channels organized by channel type. A dict is returned
    with 'eeg', 'grad' or 'mag' as keys and lists of channels as values.
    Key-value pair is omitted if no channels present, e.g. if there is no
    eeg channels, only 'grad' and 'mag' keys can be present.

    Parameters
    ----------
    info : mne.Info
        The info structure.

    Returns
    -------
    dict
        A dict of channels organized by type.
    """
    channels = {}
    grad_idxs = mne.pick_types(info, meg='grad', eeg=False)
    if grad_idxs.size > 0:
        channels['grad'] = [ch_name for idx, ch_name 
                            in enumerate(info['ch_names'])
                            if idx in grad_idxs]
    mag_idxs = mne.pick_types(info, meg='mag', eeg=False)
    if mag_idxs.size > 0:
        channels['mag'] = [ch_name for idx, ch_name 
                           in enumerate(info['ch_names'])
                           if idx in mag_idxs]
    eeg_idxs = mne.pick_types(info, meg=False, eeg=True)
    if eeg_idxs.size > 0:
        channels['eeg'] = [ch_name for idx, ch_name 
                           in enumerate(info['ch_names'])
                           if idx in eeg_idxs]

    return channels


def get_triplet_from_mag(ch_name):
    """Get the triplet (one mag, two grads) from mag by name hacking.

    Parameters
    ----------
    ch_name : str
        Name of mag channel, e.g 'MEG 1231'.

    Returns
    -------
    list
        List of three channels, e.g ['MEG 1231', 'MEG 1232', 'MEG 1233']
    """ 
    return [ch_name, ch_name[:-1] + '2', ch_name[:-1] + '3']


def pairless_grads(ch_names):
    """Returns indexes of channels for which the first three numbers of the name
    are present only once. 

    This means that if ['MEG 1232', 'MEG 1332', 'MEG 1333'] is given,
    then [0] should be returned.

    Parameters
    ----------
    ch_names : list
        List of channel names, e.g. info['ch_names']

    Returns
    -------
    list
        list of indexes to input array where the channels do not have a pair.

    """
    stems = [name[:-1] for name in ch_names]
    only_once = [stem for stem in stems if stems.count(stem) == 1]
    ch_idxs = [name_idx for name_idx, name in enumerate(ch_names) if name[:-1] in only_once]
    return ch_idxs


def clean_names(names):
    """Removes whitespace from channel names, useful sometimes when comparing
    lists of channels.

    Parameters
    ----------
    names : list
        List of channel names

    Returns
    -------
    list
        Returns list of channel names without any whitespace.

    """
    return [name.replace(' ', '') for name in names]


def iterate_topography(fig, info, ch_names, on_pick):
    """Iterator that wraps the mne.viz.iter_topography and yields a axes in correct
    location for each channel.

    The main reason for this function is historical, as info['ch_names'] did not 
    necessarily match with ch_names stored in a data object before. Thus this returns 
    idx in ch_names in addition to idx in info['ch_names']. Nowadays
    the info is stored with data objects, and thus this feature is not needed.

    It is probably fine to keep this function in the future, however, but should
    simplify.

    Parameters
    ----------
    fig : matplotlib.figure.Figure
        Figure to plot into.
    info : mne.Info
        Info structure that contains channel information (locations and names).
    ch_names : list
        List of channel names expected to be in the data.
    on_pick : function
        A function that is called if a subplot is clicked.

    Yields 
    ------
    tuple
        A tuple of (axes, idx in info['ch_names'], idx in ch_names)
    """
    ch_names = clean_names(ch_names)
    info_names = clean_names(info['ch_names'])

    def handler(ax, info_idx):
        names_idx = ch_names.index(info_names[info_idx])
        on_pick(ax, info_idx, names_idx)

    for ax, info_idx in mne.viz.iter_topography(info, fig=fig,
                                           fig_facecolor='white',
                                           axis_spinecolor='white',
                                           axis_facecolor='white',
                                           on_pick=handler):
        try: 
            names_idx = ch_names.index(info_names[info_idx])
            yield ax, info_idx, names_idx
        except ValueError as exc:
            continue


def average_to_channel_groups(data, info, ch_names, channel_groups):
    """Averages data (first dimension representing the channels) to channel groups. 

    Gets types from info but indices from ch_names.

    Parameters
    ----------
    data : np.array
        Data as a numpy array, with shape (n_channels, ...)
    info : mne.Info
        Info structure containing the channel information
    ch_names : list
        List of channel names, dimension should match data.
    channel_groups : dict
        A nested dictionary containing, where the first level is 'meg' or 'eeg', and the
        second level has the channel group names, e.g 'Left-frontal', as keys and 
        lists of channels are values.

    Returns
    -------
    list 
        A list of tuple-labels (such as ('eeg', 'Left-frontal')).
    np.array
        The matching averaged data.
    """
    chs_by_type = get_channels_by_type(info)

    ch_names = clean_names(ch_names)

    averaged_data = []
    data_labels = []

    for ch_type, chs in chs_by_type.items():
        chs = clean_names(ch_names)
        ch_names_in_chs = [ch_name for ch_name in ch_names if ch_name in chs]

        if ch_type in ['grad', 'mag']:
            ch_groups = channel_groups['meg']
        else:
            ch_groups = channel_groups['eeg']

        for ch_group, ch_group_channels in ch_groups.items():
            ch_group_channels = clean_names(ch_group_channels)
            final_ch_names = [ch_name for ch_name in ch_names_in_chs if ch_name 
                              in ch_group_channels]

            # leave here if for example ch names in ch_groups and info don't match
            if not final_ch_names:
                continue

            ch_idxs = [idx for idx, ch_name in enumerate(ch_names) if 
                       ch_name in final_ch_names]

            # calculate average
            data_in_chs = [data[ch_idx] for ch_idx in ch_idxs]
            if ch_type == 'grad':
                ch_average = np.sqrt(
                    np.sum(np.array(data_in_chs)**2, axis=0))
            else:
                ch_average = np.mean(data_in_chs, axis=0)

            averaged_data.append(ch_average)
            data_labels.append((ch_type, ch_group))

    averaged_data = np.array(averaged_data)

    return data_labels, averaged_data




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'shortMessageBoxUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_shortMessageBox(object):
    def setupUi(self, shortMessageBox):
        shortMessageBox.setObjectName("shortMessageBox")
        shortMessageBox.resize(582, 117)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(shortMessageBox.sizePolicy().hasHeightForWidth())
        shortMessageBox.setSizePolicy(sizePolicy)
        self.gridLayout = QtWidgets.QGridLayout(shortMessageBox)
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        self.pushButtonClose = QtWidgets.QPushButton(shortMessageBox)
        self.pushButtonClose.setObjectName("pushButtonClose")
        self.gridLayout.addWidget(self.pushButtonClose, 1, 1, 1, 1)
        self.scrollArea = QtWidgets.QScrollArea(shortMessageBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 562, 68))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.labelMessage = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelMessage.sizePolicy().hasHeightForWidth())
        self.labelMessage.setSizePolicy(sizePolicy)
        self.labelMessage.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.labelMessage.setWordWrap(True)
        self.labelMessage.setObjectName("labelMessage")
        self.gridLayout_2.addWidget(self.labelMessage, 0, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 2)

        self.retranslateUi(shortMessageBox)
        self.pushButtonClose.clicked.connect(shortMessageBox.close)
        QtCore.QMetaObject.connectSlotsByName(shortMessageBox)

    def retranslateUi(self, shortMessageBox):
        _translate = QtCore.QCoreApplication.translate
        shortMessageBox.setWindowTitle(_translate("shortMessageBox", "Meggie"))
        self.pushButtonClose.setText(_translate("shortMessageBox", "Close"))
        self.labelMessage.setText(_translate("shortMessageBox", "The was a cat behind the all mighty bird singing hallelujah. Who knew it would be so devastating if not all of us? The cries in the woods, howls of the never ending human culture vanishing into thin q numbers."))




import tempfile
import os

import mne
from mne.time_frequency import tfr_morlet

from meggie.datatypes.tfr.tfr import TFR

from meggie.utilities.events import find_events
from meggie.utilities.filemanager import ensure_folders

def test_tfr():
    with tempfile.TemporaryDirectory() as dirpath:

        sample_folder = mne.datasets.sample.data_path()
        sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

        name = 'TestTFR'
        cond_name = 'cond_1'
        tfr_dir = os.path.join(dirpath, 'tfr')

        raw = mne.io.read_raw_fif(sample_fname, preload=True)
        events = find_events(raw, id_=1)
        freqs = [8, 9, 10, 11, 12]
        n_cycles = 2
        mne_tfr = tfr_morlet(
            mne.Epochs(raw, events),
            freqs, n_cycles,
            return_itc=False)

        # As meggie-style tfrs can be based on multiple mne TFR objects,
        # content is dict-type. conditions-param is added to avoid accidents
        # in content loading..
        content = {cond_name: mne_tfr}
        params = {'conditions': [cond_name]}

        # Create meggie-TFR object
        # and save it to tfr directory
        tfr = TFR(name, tfr_dir, params, content=content)
        ensure_folders([tfr_dir])
        tfr.save_content()

        # Creating meggie-TFR object with same name and folder should allow
        # accessing the saved content. 
        loaded_tfr = TFR(name, tfr_dir, params)

        assert(loaded_tfr.content[cond_name].nave == 72)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'groupSelectionDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_groupSelectionDialog(object):
    def setupUi(self, groupSelectionDialog):
        groupSelectionDialog.setObjectName("groupSelectionDialog")
        groupSelectionDialog.resize(402, 605)
        self.verticalLayout = QtWidgets.QVBoxLayout(groupSelectionDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(groupSelectionDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 382, 554))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBoxGroups = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxGroups.setMinimumSize(QtCore.QSize(350, 0))
        self.groupBoxGroups.setObjectName("groupBoxGroups")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBoxGroups)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout_2.addWidget(self.groupBoxGroups, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 1, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(groupSelectionDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonCompute = QtWidgets.QPushButton(groupSelectionDialog)
        self.pushButtonCompute.setObjectName("pushButtonCompute")
        self.horizontalLayout.addWidget(self.pushButtonCompute)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(groupSelectionDialog)
        self.pushButtonCancel.clicked.connect(groupSelectionDialog.reject)
        self.pushButtonCompute.clicked.connect(groupSelectionDialog.accept)
        QtCore.QMetaObject.connectSlotsByName(groupSelectionDialog)

    def retranslateUi(self, groupSelectionDialog):
        _translate = QtCore.QCoreApplication.translate
        groupSelectionDialog.setWindowTitle(_translate("groupSelectionDialog", "Meggie - Group selection"))
        self.groupBoxGroups.setTitle(_translate("groupSelectionDialog", "Groups:"))
        self.pushButtonCancel.setText(_translate("groupSelectionDialog", "Cancel"))
        self.pushButtonCompute.setText(_translate("groupSelectionDialog", "Accept"))



""" Contains implementation for raw event info
"""
import numpy as np 

from meggie.mainwindow.dynamic import InfoAction
from meggie.utilities.events import find_stim_channel
from meggie.utilities.events import find_events


class Info(InfoAction):
    """ Shows event information on a info box.
    """

    def run(self):
        try:
            subject = self.experiment.active_subject
            if not subject:
                return ""

            raw = subject.get_raw()

            stim_ch = find_stim_channel(raw)
            if not stim_ch:
                return ""

            events = find_events(raw, stim_ch=stim_ch)
            if events is None:
                return ""

            bins = np.bincount(events[:,2])
            event_counts = dict()
            for event_id in set(events[:, 2]):
                event_counts[event_id] = bins[event_id]

            events_string = ""
            for key, value in event_counts.items():
                events_string += 'Trigger %s, %s events\n' % (str(key), str(value))

            return events_string
        except Exception as exc:
            return ""




""" Contains a class for logic of the single channel plot dialog.
"""
import logging

from PyQt5 import QtWidgets

from meggie.utilities.dialogs.singleChannelDialogUi import Ui_singleChannelDialog


class SingleChannelDialog(QtWidgets.QDialog):
    """ Contains logic for the single channel plot dialog.
    """
    def __init__(self, parent, handler, title, ch_names,
                 scalings, units, ylims, default_legend_names):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_singleChannelDialog()
        self.ui.setupUi(self)
        self.handler = handler
        self.scalings = scalings
        self.units = units
        self.ylims = ylims
        self.default_legend_names = default_legend_names

        # populate channel combobox
        for ch_name in sorted(ch_names):
            self.ui.comboBoxChannel.addItem(ch_name)

        # populate title
        self.ui.lineEditTitle.setText(title)

        # populate legend settings
        for legend_idx, legend_name in enumerate(default_legend_names):

            label_item = QtWidgets.QLabel(self.ui.groupBoxLegend)
            label_item.setText(legend_name)
            self.ui.formLayoutLegend.setWidget(legend_idx,
                QtWidgets.QFormLayout.LabelRole, label_item)

            line_edit_item = QtWidgets.QLineEdit(self.ui.groupBoxLegend)
            setattr(self.ui, 'lineEditItem_' + str(legend_idx),
                line_edit_item)
            line_edit_item.setText(legend_name)
            self.ui.formLayoutLegend.setWidget(legend_idx,
                QtWidgets.QFormLayout.FieldRole, line_edit_item)

    def on_comboBoxChannel_currentTextChanged(self, item):
        self.ui.doubleSpinBoxMin.setSuffix(' ' + self.units[item])
        self.ui.doubleSpinBoxMax.setSuffix(' ' + self.units[item])

        self.ui.doubleSpinBoxMin.setValue(self.ylims[item][0] *
                                          self.scalings[item] * 1.05)
        self.ui.doubleSpinBoxMax.setValue(self.ylims[item][1] *
                                          self.scalings[item] * 1.05)


    def accept(self):
        ymax = self.ui.doubleSpinBoxMax.value()
        ymin = self.ui.doubleSpinBoxMin.value()
        ylim = (ymin, ymax)

        ch_name = self.ui.comboBoxChannel.currentText()
        window_len = self.ui.spinBoxWindowLength.value()
        window = self.ui.comboBoxWindow.currentText()

        title = self.ui.lineEditTitle.text()

        legend = {}
        for idx in range(len(self.default_legend_names)):
            line_edit = getattr(self.ui, 'lineEditItem_' + str(idx))
            legend[self.default_legend_names[idx]] = line_edit.text()

        self.handler(ch_name, title, legend, ylim, window, window_len)

        self.close()



""" Contains create evoked action handling.
"""

from meggie.utilities.names import next_available_name
from meggie.utilities.validators import assert_arrays_same
from meggie.utilities.threading import threaded

from meggie.datatypes.evoked.evoked import Evoked

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action

from meggie.utilities.dialogs.simpleDialogMain import SimpleDialog


class CreateEvoked(Action):
    """ Allows averaging epochs to evoked
    """

    def run(self):

        selected_names = self.data['inputs']['epochs']

        if not selected_names:
            return

        if len(selected_names) == 1:
            stem = selected_names[0]
        else:
            stem = 'Evoked'

        default_name = next_available_name(
            self.experiment.active_subject.evoked.keys(), stem)

        def close_handle(subject, params):
            params['conditions'] = selected_names
            self.handler(subject, params)

        dialog = SimpleDialog(self.experiment, self.window,
                              default_name, close_handle, title='Create evoked')
        dialog.show()


    @subject_action
    def handler(self, subject, params):
        """
        """
        # check that selected epochs have similar times structure
        time_arrays = []
        for name in params['conditions']:
            epochs = subject.epochs.get(name)
            if epochs:
                time_arrays.append(epochs.content.times)
        assert_arrays_same(time_arrays)

        evokeds = {}
        for name in params['conditions']:
            epochs = subject.epochs.get(name)
            if not epochs:
                raise KeyError('No epoch collection called ' + str(name))
            mne_epochs = epochs.content

            @threaded
            def average():
                return mne_epochs.average()

            mne_evoked = average(do_meanwhile=self.window.update_ui)
            evokeds[name] = mne_evoked

            evoked_directory = subject.evoked_directory
            evoked = Evoked(params['name'], evoked_directory, 
                            params, content=evokeds)
            evoked.save_content()
            subject.add(evoked, 'evoked')




""" Contains implementation for raw rereference
"""
import logging

import matplotlib.pyplot as plt
import numpy as np

from meggie.utilities.messaging import messagebox
from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.threading import threaded

from meggie.actions.raw_rereference.dialogs.rereferencingDialogMain import RereferencingDialog

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class Rereference(Action):
    """ Shows a dialog and then allows rereferencing eeg data.
    """

    @subject_action
    def handler(self, subject, params):
        """
        """
        @threaded
        def rereference_fun():
            raw = subject.get_raw()
            if params['selection'] == 'Use average':
                raw.set_eeg_reference(ref_channels='average', 
                                      projection=False)
            elif params['selection'] == '':
                raise Exception('Empty selection')
            else:
                raw.set_eeg_reference(ref_channels=[selection])

        rereference_fun(do_meanwhile=self.window.update_ui)
        subject.rereferenced = True
        subject.save()

    def run(self):
        rereference_dialog = RereferencingDialog(
            self.window, self.experiment, self.handler)
        rereference_dialog.show()



""" Contains controlling logic for the filter.
"""

import logging

def filter_data(subject, params, preview=False):
    """
    Filters the data array in place according to parameters in paramDict.
    Depending on the parameters, the filter is one or more of
    lowpass, highpass and bandstop (notch) filter.
    """

    raw = subject.get_raw()

    if preview:
        raw = raw.copy()

    hfreq = params['low_cutoff_freq'] if params['lowpass'] else None
    lfreq = params['high_cutoff_freq'] if params['highpass'] else None
    length = params['length']
    trans_bw = params['trans_bw']

    raw.filter(l_freq=lfreq, h_freq=hfreq, filter_length=length,
               l_trans_bandwidth=trans_bw, h_trans_bandwidth=trans_bw,
               method='fft', fir_design='firwin')

    freqs = list()
    if params['bandstop1']:
        freqs.append(params['bandstop1_freq'])
    if params['bandstop2']:
        freqs.append(params['bandstop2_freq'])
    if len(freqs) > 0:
        length = params['bandstop_length']
        trans_bw = params['bandstop_transbw']

        raw.notch_filter(freqs, picks=None, filter_length=length,
                         notch_widths=params['bandstop_bw'], trans_bandwidth=trans_bw)

    if not preview:
        subject.save()

    return raw



import tempfile
import os

import mne

from meggie.datatypes.evoked.evoked import Evoked

from meggie.utilities.events import find_events
from meggie.utilities.filemanager import ensure_folders

def test_evoked():
    with tempfile.TemporaryDirectory() as dirpath:

        sample_folder = mne.datasets.sample.data_path()
        sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

        name = 'TestEvoked'
        cond_name = 'cond_1'
        evoked_dir = os.path.join(dirpath, 'evoked')

        raw = mne.io.read_raw_fif(sample_fname, preload=True)
        events = find_events(raw, id_=1)
        mne_evoked = mne.Epochs(raw, events).average()

        # As meggie-style evokeds can be based on multiple mne evoked objects,
        # content is dict-type.
        content = {cond_name: mne_evoked}

        params = {}

        # Create meggie-Evoked object
        # and save it to evoked directory
        evoked = Evoked(name, evoked_dir, params, content=content)
        ensure_folders([evoked_dir])
        evoked.save_content()

        # Creating meggie-Evoked object with same name and folder should allow
        # accessing the saved content.
        loaded_evoked = Evoked(name, evoked_dir, params)

        assert(loaded_evoked.content[cond_name].nave == 72)




"""Helper to start meggie, used by the setup.py script."""

# use Qt5Agg as TkAgg really hates threads
import matplotlib
matplotlib.use('Qt5Agg')

from meggie import mainWindowMain


def main():
    """ Run Meggie main window.
    """
    mainWindowMain.main()

# To start meggie directly from the command-line
if __name__ == '__main__':
    mainWindowMain.main()



""" Contains group average implementation.
"""

import logging

from copy import deepcopy

import mne

import numpy as np

from meggie.datatypes.spectrum.spectrum import Spectrum

from meggie.utilities.validators import assert_arrays_same
from meggie.utilities.channels import clean_names
from meggie.utilities.threading import threaded


@threaded
def group_average_spectrum(experiment, spectrum_name, groups, new_name):
    """ Computes a group average spectrum item.
    """
    # check data cohesion
    keys = []
    freq_arrays = []
    for group_key, group_subjects in groups.items():
        for subject_name in group_subjects:
            try:
                subject = experiment.subjects.get(subject_name)
                spectrum = subject.spectrum.get(spectrum_name)
                keys.append(tuple(sorted(spectrum.content.keys())))
                freq_arrays.append(tuple(spectrum.freqs))
            except Exception as exc:
                continue

    assert_arrays_same(keys, 'Conditions do not match')
    assert_arrays_same(freq_arrays, 'Freqs do not match')

    # handle channel differences
    ch_names = []
    for group_key, group_subjects in groups.items():
        for subject_name in group_subjects:
            try:
                subject = experiment.subjects.get(subject_name)
                spectrum = subject.spectrum.get(spectrum_name)
                ch_names.append(tuple(clean_names(spectrum.ch_names)))
            except Exception as exc:
                continue

    if len(set(ch_names)) != 1:
        logging.getLogger('ui_logger').info(
            "PSD's contain different sets of channels. Identifying common ones..")

        common_ch_names = list(set.intersection(*map(set, ch_names)))

        logging.getLogger('ui_logger').info(
            str(len(common_ch_names)) + ' common channels found.')
    else:
        common_ch_names = ch_names[0]

    grand_psds = {}
    for group_key, group_subjects in groups.items():
        for subject in experiment.subjects.values():
            if subject.name not in group_subjects:
                continue

            spectrum = subject.spectrum.get(spectrum_name)
            if not spectrum:
                continue

            subject_ch_names = clean_names(spectrum.ch_names) 

            for spectrum_item_key, spectrum_item in spectrum.content.items():
                grand_key = (group_key, spectrum_item_key)

                # get common channels in "subject specific space"
                idxs = [subject_ch_names.index(ch_name) for ch_name 
                        in common_ch_names]

                spectrum_item = spectrum_item[idxs]

                if grand_key not in grand_psds:
                    grand_psds[grand_key] = []
                grand_psds[grand_key].append(spectrum_item)

    grand_averages = {}
    for key, grand_psd in grand_psds.items():
        new_key = str(key[1]) + '_group_' + str(key[0])
        if len(grand_psd) == 1:
            grand_averages[new_key] = grand_psd[0].copy()
        else:
            grand_averages[new_key] = np.mean(grand_psd, axis=0)

    subject = experiment.active_subject

    try:
        spectrum = subject.spectrum.get(spectrum_name)
    except Exception as exc:
        raise Exception('Active subject should be included in the groups')

    spectrum_directory = subject.spectrum_directory

    info = spectrum.info
    common_idxs = [ch_idx for ch_idx, ch_name in enumerate(clean_names(info['ch_names']))
                   if ch_name in common_ch_names]
    info = mne.pick_info(info, sel=common_idxs)

    freqs = spectrum.freqs
    data = grand_averages

    params = deepcopy(spectrum.params)

    # individual intervals not relevant in the group item
    params.pop('intervals', None)

    params['groups'] = groups
    params['conditions'] = [elem for elem in grand_averages.keys()]

    spectrum = Spectrum(new_name, subject.spectrum_directory,
                        params, data, freqs, info)

    spectrum.save_content()
    subject.add(spectrum, 'spectrum')



""" Contains controlling logic for the tfr implementation
"""

import numpy as np
import matplotlib.pyplot as plt
import mne

from meggie.utilities.plotting import create_channel_average_plot
from meggie.utilities.channels import average_to_channel_groups


def plot_tfr_averages(subject, tfr_name, tfr_condition, 
                      blmode, blstart, blend,
                      tmin, tmax, fmin, fmax, channel_groups):
    """ Plots tfr averages.
    """

    meggie_tfr = subject.tfr[tfr_name]

    if blmode:
        bline = (blstart, blend)
        mode = blmode
    else:
        bline = None
        mode = None

    tfr = meggie_tfr.content.get(tfr_condition)

    data = tfr.data
    ch_names = meggie_tfr.ch_names

    sfreq = meggie_tfr.info['sfreq']
    times = meggie_tfr.times
    freqs = meggie_tfr.freqs

    # compared to spectrums, evoked and tse, tfr is plotted with only one condition.
    # it makes the plotting a bit simpler. we will also misuse 
    # AverageTFR object to do the heavy work.

    data_labels, averaged_data = average_to_channel_groups(
        data, meggie_tfr.info, ch_names, channel_groups)

    averages = {}
    for label_idx, label in enumerate(data_labels):
        averages[label] = averaged_data[label_idx]

    ch_types = sorted(set([label[0] for label in data_labels]))
    for ch_type in ch_types:

        ch_groups = sorted([label[1] for label in data_labels
                            if label[0] == ch_type])

        def plot_fun(ax_idx, ax):
            ch_group = ch_groups[ax_idx]
            ax.set_title(ch_group)

            info = mne.create_info(ch_names=['grand_average'], sfreq=sfreq, ch_types='mag')
            tfr = mne.time_frequency.tfr.AverageTFR(info,
                averages[(ch_type, ch_group)][np.newaxis, :], times, freqs, 1)

            # prevent interaction as no topography is involved now
            def onselect(*args, **kwargs):
                pass
            tfr._onselect = onselect

            tfr.plot(baseline=bline, mode=mode, title='', 
                     fmin=fmin, fmax=fmax, 
                     tmin=tmin, tmax=tmax, axes=ax)

        title = ' '.join([tfr_name, tfr_condition, ch_type])
        create_channel_average_plot(len(ch_groups), plot_fun, title)


def plot_tfr_topo(subject, tfr_name, tfr_condition, 
                  blmode, blstart, blend,
                  tmin, tmax, fmin, fmax, ch_type):
    """ Plots tfr topography.
    """

    meggie_tfr = subject.tfr[tfr_name]

    if blmode:
        bline = (blstart, blend)
        mode = blmode
    else:
        bline = None
        mode = None

    tfr = meggie_tfr.content.get(tfr_condition)

    if ch_type == 'eeg':
        dropped_names = [ch_name for ch_idx, ch_name in enumerate(tfr.info['ch_names'])
                         if ch_idx not in mne.pick_types(tfr.info, eeg=True, meg=False)]
    else:
        dropped_names = [ch_name for ch_idx, ch_name in enumerate(tfr.info['ch_names'])
                         if ch_idx not in mne.pick_types(tfr.info, eeg=False, meg=True)]

    tfr = tfr.copy().drop_channels(dropped_names)

    title = '{0}_{1}_{2}'.format(tfr_name, tfr_condition, ch_type)
    fig = tfr.plot_topo(show=False,
                        baseline=bline, mode=mode,
                        tmin=tmin, tmax=tmax,
                        fmin=fmin, fmax=fmax,
                        title="")

    fig.canvas.set_window_title(title)

    def onclick(event):
        """ hacky way to change title and add colorbar after creation """
        ax = plt.gca()
        fig = plt.gcf()

        channel = plt.getp(ax, 'title')
        ax.set_title('')

        title = ' '.join([tfr_name, channel])
        fig.canvas.set_window_title(title.replace(' ', '_'))
        fig.suptitle(title)

        img = ax.get_images()[0]
        plt.colorbar(mappable=img, ax=ax)

        plt.show(block=False)

    fig.canvas.mpl_connect('button_press_event', onclick)
    fig.show()




""" Contains a class for logic of add subject dialog.
"""
import os
import traceback
import logging

import mne

from PyQt5 import QtWidgets
from PyQt5 import QtCore

from meggie.mainwindow.dialogs.addSubjectDialogUi import Ui_AddSubject

from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.messaging import messagebox
from meggie.utilities.threading import threaded
from meggie.utilities.names import next_available_name


class AddSubjectDialog(QtWidgets.QDialog):
    """ Contains the logic for add subject dialog.
    """

    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_AddSubject()
        self.ui.setupUi(self)

        self.parent = parent

    
    def accept(self):
        n_successful = 0
        for i in range(self.ui.listWidgetFileNames.count()):
            item = self.ui.listWidgetFileNames.item(i)
            raw_path = item.text()
            basename = os.path.basename(raw_path)
            subject_name = basename.split('.')[0]

            experiment = self.parent.experiment
            old_names = experiment.subjects.keys()

            try:
                subject_name = next_available_name(old_names, subject_name)

                @threaded
                def _create_subject():
                    experiment.create_subject(subject_name, raw_path)
                
                _create_subject(do_meanwhile=self.parent.update_ui)
                n_successful += 1

            except Exception as exc:
                logging.getLogger('ui_logger').exception('')

        try:
            self.parent.experiment.save_experiment_settings()
        except Exception as exc:
            exc_messagebox(self, exc)
            return

        n_total = self.ui.listWidgetFileNames.count()

        if n_total != n_successful:
            message = ("Only {0} / {1} subjects added successfully. "
                       "Please check console below for details.")
            messagebox(self.parent, message.format(n_successful, n_total))

        message = ('{0} / {1} subjects added successfully.').format(
            n_successful, n_total)
        logging.getLogger('ui_logger').info(message)

        self.parent.initialize_ui()
        self.close()

    def on_pushButtonBrowse_clicked(self, checked=None):
        if checked is None:
            return

        mne_supported = mne.io._read_raw.supported

        all_ext = []
        all_items = []
        for row in mne_supported.items():
            ext = row[0]
            name = row[1].__name__.split('_')[-1]
            all_ext.append(ext)
            all_items.append(name + ' files (*' + ext + ')')

        filter_string = 'All supported (*' + ' *'.join(all_ext) + ');'';'
        filter_string = filter_string + ';'';'.join(all_items)

        self.fnames = QtWidgets.QFileDialog.getOpenFileNames(
            self,
            'Select one or more files to open.', 
            os.path.expanduser("~"),
            filter_string)[0]

        if len(self.fnames) > 0:
            for name in self.fnames:
                name = QtCore.QDir.toNativeSeparators(name)
                item = QtWidgets.QListWidgetItem()
                item.setText(name)
                if len(self.ui.listWidgetFileNames.findItems(
                       name, QtCore.Qt.MatchExactly)) > 0:
                    continue
                self.ui.listWidgetFileNames.addItem(item)

    def on_pushButtonRemove_clicked(self, checked=None):
        if checked is None:
            return
        for item in self.ui.listWidgetFileNames.selectedItems():
            i = self.ui.listWidgetFileNames.indexFromItem(item)
            row = i.row()
            self.ui.listWidgetFileNames.takeItem(row)

    def on_listWidgetFileNames_itemSelectionChanged(self):
        items = self.ui.listWidgetFileNames.selectedItems()
        if len(items) > 0:
            self.ui.pushButtonRemove.setEnabled(True)
        else:
            self.ui.pushButtonRemove.setEnabled(False)



import tempfile
import os

import mne

from meggie.datatypes.epochs.epochs import Epochs

from meggie.utilities.events import find_events
from meggie.utilities.filemanager import ensure_folders

def test_epochs():
    with tempfile.TemporaryDirectory() as dirpath:

        sample_folder = mne.datasets.sample.data_path()
        sample_fname = os.path.join(sample_folder, 'MEG', 'sample', 'sample_audvis_raw.fif')

        raw = mne.io.read_raw_fif(sample_fname, preload=True)
        events = find_events(raw, id_=1)
        mne_epochs = mne.Epochs(raw, events)

        name = 'TestEpochs'
        epochs_dir = os.path.join(dirpath, 'epochs')

        params = {}

        # Create meggie-Epochs object with mne-Epochs stored within
        # and save it to epochs directory
        epochs = Epochs(name, epochs_dir, params, content=mne_epochs)
        ensure_folders([epochs_dir])
        epochs.save_content()

        # Creating meggie-Epochs object with same name and folder should allow
        # accessing the saved content
        loaded_epochs = Epochs(name, epochs_dir, params)

        assert(epochs.count == loaded_epochs.count == 72)



import tempfile
import os
import shutil
import numpy as np

from meggie.experiment import Experiment
from meggie.utilities.filemanager import create_timestamped_folder
from meggie.utilities.filemanager import save_csv
from meggie.utilities.filemanager import load_csv


def test_create_timestamped_folder():
    with tempfile.TemporaryDirectory() as dirpath:

        name = 'test_experiment'
        author = ''
        path = os.path.join(dirpath, name)
        experiment = Experiment(name, author, path)

        create_timestamped_folder(experiment)

        contents = os.listdir(os.path.join(experiment.path, 'output'))
        assert(len(contents) > 0)


def test_save_and_load_csv():
    with tempfile.TemporaryDirectory() as dirpath:

        filepath = os.path.join(dirpath, 'data.csv')

        data = np.array([[1,2,3], [4,5,6]])
        column_names = ['A', 'B', 'C']
        row_descs = [('X', 'Kissa'), ('Y', 'Koira')]

        save_csv(filepath, data, column_names, row_descs)

        loaded_column_names, loaded_row_descs, loaded_data = load_csv(filepath)

        assert(loaded_column_names == column_names)
        assert(loaded_row_descs[0] == row_descs[0])
        assert(loaded_row_descs[1] == row_descs[1])
        assert(np.array_equal(loaded_data, data))




""" Contains implementation for raw resample.
"""
import logging

import matplotlib.pyplot as plt
import numpy as np

from meggie.utilities.messaging import messagebox
from meggie.utilities.messaging import exc_messagebox
from meggie.utilities.threading import threaded

from meggie.actions.raw_resample.dialogs.resamplingDialogMain import ResamplingDialog

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action


class Resample(Action):
    """ Shows a dialog for getting parameters and allows
    resampling data of the subject.
    """

    @subject_action
    def handler(self, subject, params):
        """
        """
        @threaded
        def resample_fun():
            subject.get_raw().resample(params['rate'])

        resample_fun(do_meanwhile=self.window.update_ui)
        subject.save()

    def run(self):
        resampling_dialog = ResamplingDialog(
            self.window, self.experiment, self.handler)
        resampling_dialog.show()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'TFROutputOptionsUi.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TFROutputOptions(object):
    def setupUi(self, TFROutputOptions):
        TFROutputOptions.setObjectName("TFROutputOptions")
        TFROutputOptions.resize(546, 577)
        self.gridLayout_2 = QtWidgets.QGridLayout(TFROutputOptions)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.scrollArea = QtWidgets.QScrollArea(TFROutputOptions)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 514, 541))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.doubleSpinBoxTimeStart = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.doubleSpinBoxTimeStart.setSingleStep(0.1)
        self.doubleSpinBoxTimeStart.setObjectName("doubleSpinBoxTimeStart")
        self.gridLayout_3.addWidget(self.doubleSpinBoxTimeStart, 0, 1, 1, 1)
        self.labelTimeStart = QtWidgets.QLabel(self.groupBox)
        self.labelTimeStart.setObjectName("labelTimeStart")
        self.gridLayout_3.addWidget(self.labelTimeStart, 0, 0, 1, 1)
        self.labelTimeEnd = QtWidgets.QLabel(self.groupBox)
        self.labelTimeEnd.setObjectName("labelTimeEnd")
        self.gridLayout_3.addWidget(self.labelTimeEnd, 1, 0, 1, 1)
        self.doubleSpinBoxTimeEnd = QtWidgets.QDoubleSpinBox(self.groupBox)
        self.doubleSpinBoxTimeEnd.setMinimum(0.0)
        self.doubleSpinBoxTimeEnd.setSingleStep(0.1)
        self.doubleSpinBoxTimeEnd.setProperty("value", 0.0)
        self.doubleSpinBoxTimeEnd.setObjectName("doubleSpinBoxTimeEnd")
        self.gridLayout_3.addWidget(self.doubleSpinBoxTimeEnd, 1, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBox, 4, 0, 1, 1)
        self.groupBoxBaseline = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxBaseline.setObjectName("groupBoxBaseline")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBoxBaseline)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.labelBaselineCorrection = QtWidgets.QLabel(self.groupBoxBaseline)
        self.labelBaselineCorrection.setObjectName("labelBaselineCorrection")
        self.gridLayout_9.addWidget(self.labelBaselineCorrection, 0, 0, 1, 1)
        self.doubleSpinBoxBaselineStart = QtWidgets.QDoubleSpinBox(self.groupBoxBaseline)
        self.doubleSpinBoxBaselineStart.setSingleStep(0.1)
        self.doubleSpinBoxBaselineStart.setObjectName("doubleSpinBoxBaselineStart")
        self.gridLayout_9.addWidget(self.doubleSpinBoxBaselineStart, 2, 1, 1, 1)
        self.doubleSpinBoxBaselineEnd = QtWidgets.QDoubleSpinBox(self.groupBoxBaseline)
        self.doubleSpinBoxBaselineEnd.setSingleStep(0.1)
        self.doubleSpinBoxBaselineEnd.setObjectName("doubleSpinBoxBaselineEnd")
        self.gridLayout_9.addWidget(self.doubleSpinBoxBaselineEnd, 3, 1, 1, 1)
        self.comboBoxBaselineMode = QtWidgets.QComboBox(self.groupBoxBaseline)
        self.comboBoxBaselineMode.setObjectName("comboBoxBaselineMode")
        self.comboBoxBaselineMode.addItem("")
        self.comboBoxBaselineMode.addItem("")
        self.comboBoxBaselineMode.addItem("")
        self.comboBoxBaselineMode.addItem("")
        self.comboBoxBaselineMode.addItem("")
        self.gridLayout_9.addWidget(self.comboBoxBaselineMode, 1, 1, 1, 1)
        self.labelBaselineStart = QtWidgets.QLabel(self.groupBoxBaseline)
        self.labelBaselineStart.setObjectName("labelBaselineStart")
        self.gridLayout_9.addWidget(self.labelBaselineStart, 2, 0, 1, 1)
        self.labelBaselineEnd = QtWidgets.QLabel(self.groupBoxBaseline)
        self.labelBaselineEnd.setObjectName("labelBaselineEnd")
        self.gridLayout_9.addWidget(self.labelBaselineEnd, 3, 0, 1, 1)
        self.labelBaselineMode = QtWidgets.QLabel(self.groupBoxBaseline)
        self.labelBaselineMode.setObjectName("labelBaselineMode")
        self.gridLayout_9.addWidget(self.labelBaselineMode, 1, 0, 1, 1)
        self.checkBoxBaselineCorrection = QtWidgets.QCheckBox(self.groupBoxBaseline)
        self.checkBoxBaselineCorrection.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.checkBoxBaselineCorrection.setText("")
        self.checkBoxBaselineCorrection.setChecked(True)
        self.checkBoxBaselineCorrection.setObjectName("checkBoxBaselineCorrection")
        self.gridLayout_9.addWidget(self.checkBoxBaselineCorrection, 0, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBoxBaseline, 2, 0, 1, 1)
        self.groupBoxGrouping = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxGrouping.setObjectName("groupBoxGrouping")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxGrouping)
        self.formLayout.setObjectName("formLayout")
        self.labelChannels = QtWidgets.QLabel(self.groupBoxGrouping)
        self.labelChannels.setObjectName("labelChannels")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.labelChannels)
        self.radioButtonAllChannels = QtWidgets.QRadioButton(self.groupBoxGrouping)
        self.radioButtonAllChannels.setChecked(True)
        self.radioButtonAllChannels.setObjectName("radioButtonAllChannels")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.radioButtonAllChannels)
        self.radioButtonChannelAverages = QtWidgets.QRadioButton(self.groupBoxGrouping)
        self.radioButtonChannelAverages.setObjectName("radioButtonChannelAverages")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.radioButtonChannelAverages)
        self.gridLayout_4.addWidget(self.groupBoxGrouping, 6, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem, 7, 0, 1, 1)
        self.groupBoxFrequency = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxFrequency.setObjectName("groupBoxFrequency")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBoxFrequency)
        self.gridLayout.setObjectName("gridLayout")
        self.labelFrequencyMax = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyMax.setObjectName("labelFrequencyMax")
        self.gridLayout.addWidget(self.labelFrequencyMax, 1, 0, 1, 1)
        self.labelFrequencyMin = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyMin.setObjectName("labelFrequencyMin")
        self.gridLayout.addWidget(self.labelFrequencyMin, 0, 0, 1, 1)
        self.doubleSpinBoxFrequencyMin = QtWidgets.QDoubleSpinBox(self.groupBoxFrequency)
        self.doubleSpinBoxFrequencyMin.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyMin.setObjectName("doubleSpinBoxFrequencyMin")
        self.gridLayout.addWidget(self.doubleSpinBoxFrequencyMin, 0, 1, 1, 1)
        self.doubleSpinBoxFrequencyMax = QtWidgets.QDoubleSpinBox(self.groupBoxFrequency)
        self.doubleSpinBoxFrequencyMax.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyMax.setObjectName("doubleSpinBoxFrequencyMax")
        self.gridLayout.addWidget(self.doubleSpinBoxFrequencyMax, 1, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBoxFrequency, 3, 0, 1, 1)
        self.groupBoxCondition = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxCondition.setObjectName("groupBoxCondition")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxCondition)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.labelCondition = QtWidgets.QLabel(self.groupBoxCondition)
        self.labelCondition.setObjectName("labelCondition")
        self.gridLayout_5.addWidget(self.labelCondition, 0, 0, 1, 1)
        self.comboBoxCondition = QtWidgets.QComboBox(self.groupBoxCondition)
        self.comboBoxCondition.setObjectName("comboBoxCondition")
        self.gridLayout_5.addWidget(self.comboBoxCondition, 0, 1, 1, 1)
        self.gridLayout_4.addWidget(self.groupBoxCondition, 5, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_2.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(TFROutputOptions)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonOk = QtWidgets.QPushButton(TFROutputOptions)
        self.pushButtonOk.setObjectName("pushButtonOk")
        self.horizontalLayout.addWidget(self.pushButtonOk)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(TFROutputOptions)
        self.checkBoxBaselineCorrection.toggled['bool'].connect(self.comboBoxBaselineMode.setEnabled)
        self.checkBoxBaselineCorrection.toggled['bool'].connect(self.doubleSpinBoxBaselineStart.setEnabled)
        self.checkBoxBaselineCorrection.toggled['bool'].connect(self.doubleSpinBoxBaselineEnd.setEnabled)
        self.pushButtonCancel.clicked.connect(TFROutputOptions.reject)
        self.pushButtonOk.clicked.connect(TFROutputOptions.accept)
        QtCore.QMetaObject.connectSlotsByName(TFROutputOptions)
        TFROutputOptions.setTabOrder(self.scrollArea, self.checkBoxBaselineCorrection)
        TFROutputOptions.setTabOrder(self.checkBoxBaselineCorrection, self.comboBoxBaselineMode)
        TFROutputOptions.setTabOrder(self.comboBoxBaselineMode, self.doubleSpinBoxBaselineStart)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxBaselineStart, self.doubleSpinBoxBaselineEnd)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxBaselineEnd, self.doubleSpinBoxFrequencyMin)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxFrequencyMin, self.doubleSpinBoxFrequencyMax)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxFrequencyMax, self.doubleSpinBoxTimeStart)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxTimeStart, self.doubleSpinBoxTimeEnd)
        TFROutputOptions.setTabOrder(self.doubleSpinBoxTimeEnd, self.comboBoxCondition)
        TFROutputOptions.setTabOrder(self.comboBoxCondition, self.radioButtonAllChannels)
        TFROutputOptions.setTabOrder(self.radioButtonAllChannels, self.radioButtonChannelAverages)
        TFROutputOptions.setTabOrder(self.radioButtonChannelAverages, self.pushButtonCancel)
        TFROutputOptions.setTabOrder(self.pushButtonCancel, self.pushButtonOk)

    def retranslateUi(self, TFROutputOptions):
        _translate = QtCore.QCoreApplication.translate
        TFROutputOptions.setWindowTitle(_translate("TFROutputOptions", "Meggie - Output options"))
        self.groupBox.setTitle(_translate("TFROutputOptions", "Time settings:"))
        self.labelTimeStart.setText(_translate("TFROutputOptions", "Start (s):"))
        self.labelTimeEnd.setText(_translate("TFROutputOptions", "End (s):"))
        self.groupBoxBaseline.setTitle(_translate("TFROutputOptions", "Baseline:"))
        self.labelBaselineCorrection.setText(_translate("TFROutputOptions", "Baseline correction:"))
        self.comboBoxBaselineMode.setItemText(0, _translate("TFROutputOptions", "mean"))
        self.comboBoxBaselineMode.setItemText(1, _translate("TFROutputOptions", "ratio"))
        self.comboBoxBaselineMode.setItemText(2, _translate("TFROutputOptions", "logratio"))
        self.comboBoxBaselineMode.setItemText(3, _translate("TFROutputOptions", "percent"))
        self.comboBoxBaselineMode.setItemText(4, _translate("TFROutputOptions", "zscore"))
        self.labelBaselineStart.setText(_translate("TFROutputOptions", "Baseline start (s):"))
        self.labelBaselineEnd.setText(_translate("TFROutputOptions", "Baseline end (s):"))
        self.labelBaselineMode.setText(_translate("TFROutputOptions", "Baseline mode:"))
        self.groupBoxGrouping.setTitle(_translate("TFROutputOptions", "Grouping:"))
        self.labelChannels.setText(_translate("TFROutputOptions", "Type:"))
        self.radioButtonAllChannels.setText(_translate("TFROutputOptions", "All channels"))
        self.radioButtonChannelAverages.setText(_translate("TFROutputOptions", "Channel averages"))
        self.groupBoxFrequency.setTitle(_translate("TFROutputOptions", "Frequency settings:"))
        self.labelFrequencyMax.setText(_translate("TFROutputOptions", "Maximum frequency (Hz):"))
        self.labelFrequencyMin.setText(_translate("TFROutputOptions", "Minimum frequency (Hz):"))
        self.groupBoxCondition.setTitle(_translate("TFROutputOptions", "Condition:"))
        self.labelCondition.setText(_translate("TFROutputOptions", "Select condition:"))
        self.pushButtonCancel.setText(_translate("TFROutputOptions", "Cancel"))
        self.pushButtonOk.setText(_translate("TFROutputOptions", "Accept"))




""" Contains a class for logic of the short message box.
"""
from PyQt5 import QtWidgets

from meggie.utilities.dialogs.shortMessageBoxUi import Ui_shortMessageBox


class shortMessageBox(QtWidgets.QDialog):
    """ Contains logic for the short message box.
    """
    def __init__(self, message, parent):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_shortMessageBox()
        self.ui.setupUi(self)

        self.setWindowTitle('Meggie - Info')
        self.ui.labelMessage.setText(message)




"""Functions for plotting purposes.
"""

import itertools

import matplotlib.pyplot as plt
from matplotlib.lines import Line2D


def color_cycle(n):
    """Returns n color strings by cycling

    Parameters
    ----------
    n : int
        Length of the color list.

    Returns
    -------
    list 
        The colors.

    """
    cycler = itertools.cycle(['b', 'r', 'g', 'y', 'm', 'c', 'k', 'pink'])
    return list(itertools.islice(cycler, n))


def create_channel_average_plot(n_plots, plot_fun, title, legend=None):
    """Creates a figure with proper subplot layout for channel averages, 
    and calls plot_fun to fill each of the axes.

    Parameters
    ----------
    n_plots : int
        The number of subplots.
    plot_fun : function
        The function that is called for each subplot.
    title : str
        The title for the figure and the window.
    legend : list, optional
        A list of condition-color tuples. If None, no legend is drawn.

    Returns
    -------
    matplotlib.figure.Figure
        The figure.
    """
    ncols = min(4, n_plots)
    nrows = int((n_plots - 1) / ncols + 1)

    fig, axes = plt.subplots(nrows=nrows, ncols=ncols, squeeze=False,
                             constrained_layout=True)
    fig.set_size_inches(15, int(10*(nrows/ncols)))

    for ax_idx in range(ncols*nrows):
        ax = axes[ax_idx // ncols, ax_idx % ncols]
        if ax_idx >= n_plots:
            ax.axis('off')
            continue
        plot_fun(ax_idx, ax)

    if legend is not None:
        lines = []
        texts = []
        for condition, color in legend:
            lines.append(Line2D([0], [0], color=color))
            texts.append(condition)
        fig.legend(lines, texts)

    fig.suptitle(title)
    fig.canvas.set_window_title(title.replace(' ', '_'))

    return fig

